Package: irrCAC
Authors@R: person(given = c("Kilem", "L."),
                  family = "Gwet",
                  role = c("aut", "cre"),
                  email = "gwet@agreestat.com")
Version: 1.4
Date: 2026-04-26
Title: Computing the Extent of Agreement among Raters with
        Chance-Corrected Agreement Coefficient (CAC)
Description: Contains a series of R functions for calculating various chance-corrected agreement 
  coefficients (CAC) among 2 or more raters. Among the CAC coefficients covered are Cohen's kappa, Conger's kappa,
  Fleiss' kappa, Brennan-Prediger coefficient, Gwet's AC1/AC2 coefficients, and Krippendorff's alpha. Multiple
  sets of weights are proposed for computing weighted analyses. Also included in this package is Bangdiwala's B
  coefficient.
Depends: R (>= 3.5.0)
Encoding: UTF-8
Imports: dplyr, magrittr, tidyr, stringr, tibble
Language: en-US
LazyData: true
License: GPL (>= 2)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, spelling
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-26 19:33:14 UTC; GWET-ASUS-14
Author: Kilem L. Gwet [aut, cre]
Maintainer: Kilem L. Gwet <gwet@agreestat.com>
Repository: CRAN
Date/Publication: 2026-04-27 07:20:08 UTC
Built: R 4.5.2; ; 2026-04-27 09:38:02 UTC; unix

# Generated by roxygen2: do not edit by hand

export(altman.bf)
export(bangdiwala.table)
export(bangdiwala2RR.fn)
export(bipolar.weights)
export(bp.coeff.dist)
export(bp.coeff.raw)
export(bp2.table)
export(circular.weights)
export(conger.kappa.raw)
export(fleiss.bf)
export(fleiss.kappa.dist)
export(fleiss.kappa.raw)
export(freq.supp.fn)
export(gwet.ac1.dist)
export(gwet.ac1.raw)
export(gwet.ac1.table)
export(identity.weights)
export(kappa2.table)
export(krippen.alpha.dist)
export(krippen.alpha.raw)
export(krippen2.table)
export(landis.koch.bf)
export(linear.weights)
export(long2wide.fn)
export(ordinal.weights)
export(pa.coeff.dist)
export(pa.coeff.raw)
export(pa2.table)
export(quadratic.weights)
export(radical.weights)
export(ratio.weights)
export(scott2.table)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,count)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(magrittr,"%>%")
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,setNames)
importFrom(stringr,str_trim)
importFrom(tibble,as_tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_wider)

    library(irrCAC)
    #source("C:\\Users\\GWET-ASUS-14\\Documents\\Cloud Computing\\RpackagesWork\\irrCAC\\R\\agree.coeff2.r")

# Installation

devtools::install\_github(“kgwet/irrCAC”)

# Abstract

The **irrCAC** is an R package that provides several functions for
calculating various chance-corrected agreement coefficients. This
package closely follows the general framework of inter-rater reliability
assessment presented by Gwet (2014). A similar package was developed for
STATA users by Klein (2018).

The functions included in this package can handle 3 types of input data:
(1) the contingency table, (2) the distribution of raters by subject and
by category, (3) the raw data, which is essentially a plain dataset
where each row represents a subject and each column, the ratings
associated with one rater. The list of all datasets contained in this
package can be listed as follows:

      data(package="irrCAC")

# Computing Agreement Coefficients

## Computing agreement Coefficients from Contingency tables

**cont3x3abstractors** is one of 2 datasets included in this package and
that contains rating data from 2 raters organized in the form of a
contingency table. The following r script shows how to compute Cohen’s
kappa, Scott’s Pi, Gwet’s AC<sub>1</sub>, Brennan-Prediger,
Krippendorff’s alpha, Bangdiwala’s B, and the percent agreement
coefficients from this dataset.

      cont3x3abstractors
    #>         Ectopic AIU NIU
    #> Ectopic      13   0   0
    #> AIU           0  20   7
    #> NIU           0   4  56
      kappa2.table(cont3x3abstractors)
    #>      coeff.name coeff.val   coeff.se     coeff.ci coeff.pval
    #> 1 Cohen's Kappa 0.7964094 0.05891072 (0.68,0.913)      0e+00
      scott2.table(cont3x3abstractors)
    #>   coeff.name coeff.val   coeff.se      coeff.ci coeff.pval
    #> 1 Scott's Pi 0.7962397 0.05905473 (0.679,0.913)      0e+00
      gwet.ac1.table(cont3x3abstractors)
    #>   coeff.name coeff.val   coeff.se      coeff.ci coeff.pval
    #> 1 Gwet's AC1 0.8493305 0.04321747 (0.764,0.935)      0e+00
      bp2.table(cont3x3abstractors)
    #>         coeff.name coeff.val   coeff.se      coeff.ci coeff.pval
    #> 1 Brennan-Prediger     0.835 0.04693346 (0.742,0.928)      0e+00
      krippen2.table(cont3x3abstractors)
    #>             coeff.name coeff.val   coeff.se     coeff.ci coeff.pval
    #> 1 Krippendorff's Alpha 0.7972585 0.05905473 (0.68,0.914)      0e+00
      bangdiwala.table(cont3x3abstractors)
    #>       coeff.name coeff.val   coeff.se      coeff.ci coeff.pval
    #> 1 Bangdiwala's B 0.8059604 0.05449465 (0.698,0.914)      0e+00
      pa2.table(cont3x3abstractors)
    #>          coeff.name coeff.val   coeff.se      coeff.ci coeff.pval
    #> 1 Percent Agreement      0.89 0.03128898 (0.828,0.952)      0e+00

Suppose that you only want to obtain Gwet’s AC<sub>1</sub> coefficient,
but don’t care about the associated precision measures such as the
standard error, confidence intervals or p-values. You can accomplish
this as follows:

      ac1 <- gwet.ac1.table(cont3x3abstractors)$coeff.val

Then use the variable ac1 to obtain AC<sub>1</sub> = 0.849.

Another contingency table included in this package is named
**cont3x3abstractors**. You may use it to experiment with the r
functions listed above.

## Computing agreement coefficients from the distribution of raters by subject & category

Included in this package is a small dataset named **distrib.6raters**,
which contains the distribution of 6 raters by subject and category.
Each row represents a subject (i.e. a psychiatric patient) and the
number of raters (i.e. psychiatrists) who classified it into each
category used in the inter-rater reliability study. Here is the dataset
and how it can be used to compute the various agreement coefficients:

    distrib.6raters
    #>    Depression Personality.Disorder Schizophrenia Neurosis Other
    #> 1           0                    0             0        6     0
    #> 2           0                    3             0        0     3
    #> 3           0                    1             4        0     1
    #> 4           0                    0             0        0     6
    #> 5           0                    3             0        3     0
    #> 6           2                    0             4        0     0
    #> 7           0                    0             4        0     2
    #> 8           2                    0             3        1     0
    #> 9           2                    0             0        4     0
    #> 10          0                    0             0        0     6
    #> 11          1                    0             0        5     0
    #> 12          1                    1             0        4     0
    #> 13          0                    3             3        0     0
    #> 14          1                    0             0        5     0
    #> 15          0                    2             0        3     1
    gwet.ac1.dist(distrib.6raters)
    #> Warning: Returning more (or less) than 1 row per `summarise()` group was deprecated in
    #> dplyr 1.1.0.
    #> i Please use `reframe()` instead.
    #> i When switching from `summarise()` to `reframe()`, remember that `reframe()`
    #>   always returns an ungrouped data frame and adjust accordingly.
    #> i The deprecated feature was likely used in the irrCAC package.
    #>   Please report the issue to the authors.
    #> This warning is displayed once every 8 hours.
    #> Call `lifecycle::last_lifecycle_warnings()` to see where this warning was
    #> generated.
    #>   coeff.name     coeff     stderr      conf.int      p.value        pa
    #> 1 Gwet's AC1 0.4448007 0.08418757 (0.264,0.625) 5.779637e-05 0.5511111
    #>          pe  n q r.min r.max
    #> 1 0.1914815 15 5     6     6
    fleiss.kappa.dist(distrib.6raters)
    #>      coeff.name     coeff     stderr     conf.int      p.value        pa
    #> 1 Fleiss' Kappa 0.4139265 0.08119291 (0.24,0.588) 8.113618e-05 0.5511111
    #>          pe  n q r.min r.max
    #> 1 0.2340741 15 5     6     6
    krippen.alpha.dist(distrib.6raters)
    #>             coeff.name     coeff     stderr      conf.int      p.value
    #> 1 Krippendorff's Alpha 0.4204384 0.08243228 (0.244,0.597) 8.078607e-05
    #>          pa        pe  n q r.min r.max
    #> 1 0.5560988 0.2340741 15 5     6     6
    bp.coeff.dist(distrib.6raters)
    #>         coeff.name     coeff     stderr      conf.int      p.value        pa
    #> 1 Brennan-Prediger 0.4388889 0.08312142 (0.261,0.617) 5.814998e-05 0.5511111
    #>    pe  n q r.min r.max
    #> 1 0.2 15 5     6     6

Once again, you can request a single value from these functions. To get
only Krippendorff’s alpha coefficient without it’s precission measures,
you may proceed as follows:

      alpha <- krippen.alpha.dist(distrib.6raters)$coeff

The newly-created alpha variable gives the coefficient *α* = 0.4204384.

Two additional datasets that represent ratings in the form of a
distribution of raters by subject and by category, are included in this
package. These datasets are **cac.dist4cat** and **cac.dist4cat**. Note
that these 2 datasets contain more columns than needed to run the 4
functions presented in this section. Therefore, the columns associated
with the response categories must be extracted from the original
datasets before running the functions. For example, computing Gwet’s
AC<sub>1</sub> coefficient using the **cac.dist4cat** dataset should be
done as follows:

      ac1 <- gwet.ac1.dist(cac.dist4cat[,2:4])$coeff

Note that the input dataset supplied to the function is
**cac.dist4cat\[,2:4\]**. That is, only columns 2, 3, and 4 are
extracted from the original datset and used as input data. We know from
the value of the newly created variable that AC<sub>1</sub> = 0.3518903.

## Computing agreement coefficients from raw ratings

One example dataset of raw ratings included in this package is
**cac.raw4raters** and looks like this:

      cac.raw4raters
    #>    Rater1 Rater2 Rater3 Rater4
    #> 1       1      1     NA      1
    #> 2       2      2      3      2
    #> 3       3      3      3      3
    #> 4       3      3      3      3
    #> 5       2      2      2      2
    #> 6       1      2      3      4
    #> 7       4      4      4      4
    #> 8       1      1      2      1
    #> 9       2      2      2      2
    #> 10     NA      5      5      5
    #> 11     NA     NA      1      1
    #> 12     NA     NA      3     NA

As you can see, a dataset of raw ratings is merely a listing of ratings
that the raters assigned to the subjects. Each row is associated with a
single subject.Typically, the same subject would be rated by all or some
of the raters. The dataset **cac.raw4raters** contains some missing
ratings represented by the symbol NA, suggesting that some raters did
not rate all subjects. As a matter of fact, in this particular case, no
rater rated all subjects.

Here is you can compute the various agreement coefficients using the raw
ratings:

    pa.coeff.raw(cac.raw4raters)
    #> $est
    #>          coeff.name        pa pe coeff.val coeff.se  conf.int      p.value
    #> 1 Percent Agreement 0.8181818  0 0.8181818  0.12561 (0.542,1) 2.172686e-05
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11
    gwet.ac1.raw(cac.raw4raters)
    #> $est
    #>   coeff.name        pa        pe coeff.val coeff.se  conf.int      p.value
    #> 1        AC1 0.8181818 0.1903212   0.77544  0.14295 (0.461,1) 0.0001043605
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11
    fleiss.kappa.raw(cac.raw4raters)
    #> $est
    #>      coeff.name        pa        pe coeff.val coeff.se  conf.int      p.value
    #> 1 Fleiss' Kappa 0.8181818 0.2387153   0.76117  0.15302 (0.424,1) 0.0002095865
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11
    krippen.alpha.raw(cac.raw4raters)
    #> $est
    #>             coeff.name    pa   pe coeff.val coeff.se  conf.int      p.value
    #> 1 Krippendorff's Alpha 0.805 0.24   0.74342  0.14548 (0.423,1) 0.0001693123
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11
    conger.kappa.raw(cac.raw4raters)
    #> $est
    #>       coeff.name        pa        pe coeff.val coeff.se  conf.int      p.value
    #> 1 Conger's Kappa 0.8181818 0.2334252   0.76282  0.14917 (0.435,1) 0.0001683533
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11
    bp.coeff.raw(cac.raw4raters)
    #> $est
    #>         coeff.name        pa  pe coeff.val coeff.se  conf.int      p.value
    #> 1 Brennan-Prediger 0.8181818 0.2   0.77273  0.14472 (0.454,1) 0.0001187804
    #>   tot.obs     w.name
    #> 1      41 unweighted
    #> 
    #> $weights
    #>      [,1] [,2] [,3] [,4] [,5]
    #> [1,]    1    0    0    0    0
    #> [2,]    0    1    0    0    0
    #> [3,]    0    0    1    0    0
    #> [4,]    0    0    0    1    0
    #> [5,]    0    0    0    0    1
    #> 
    #> $categories
    #> [1] 1 2 3 4 5
    #> 
    #> $obs
    #>   Rater1 Rater2 Rater3 Rater4
    #> 1      9     10     11     11

Most users of this package will only be interessted in the agreement
coefficients and possibly in the related statistics such as the standard
error and p-values. In this case, you should run these functions as
follows (AC<sub>1</sub> is used here as an example. Feel free to
experiment with the other coefficients):

    ac1 <- gwet.ac1.raw(cac.raw4raters)$est
    ac1
    #>   coeff.name        pa        pe coeff.val coeff.se  conf.int      p.value
    #> 1        AC1 0.8181818 0.1903212   0.77544  0.14295 (0.461,1) 0.0001043605
    #>   tot.obs     w.name
    #> 1      41 unweighted

You can even request only the AC<sub>1</sub> coefficient estimate
0.77544. You will then proceed as follows:

    ac1 <- gwet.ac1.raw(cac.raw4raters)$est
    ac1$coeff.val
    #> [1] 0.77544

# References:

1.  Gwet, K.L. (2014). ``*Handbook of Inter-Rater Reliability*,'' 4th Edition. Advanced
    Analytics, LLC
2.  Klein, D. (2018) doi:<https://doi.org/10.1177/1536867X1801800408>.
    “Implementing a general framework for assessing interrater agreement
    in Stata,” *The Stata Journal* Volume 18, Number 4, pp. 871-901.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(irrCAC)

## -----------------------------------------------------------------------------
  identity.weights(1:3)
  radical.weights(1:3)
  linear.weights(1:3)
  ordinal.weights(1:3)
  quadratic.weights(1:3)
  circular.weights(1:3)
  bipolar.weights(1:3)

## -----------------------------------------------------------------------------
  cont3x3abstractors
  q <- nrow(cont3x3abstractors)
  kappa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  scott2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  gwet.ac1.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  bp2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  krippen2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  pa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))

## -----------------------------------------------------------------------------
  pa.coeff.raw(cac.raw4raters,weights = "quadratic")$est
  gwet.ac1.raw(cac.raw4raters,weights = "quadratic")$est
  fleiss.kappa.raw(cac.raw4raters,weights = "quadratic")$est
  krippen.alpha.raw(cac.raw4raters,weights = "quadratic")$est
  conger.kappa.raw(cac.raw4raters,weights = "quadratic")$est
  bp.coeff.raw(cac.raw4raters,weights = "quadratic")$est

## -----------------------------------------------------------------------------
  q <- ncol(distrib.6raters)
  gwet.ac1.dist(distrib.6raters,weights = quadratic.weights(1:q))
  fleiss.kappa.dist(distrib.6raters,weights = quadratic.weights(1:q))
  krippen.alpha.dist(distrib.6raters,weights = quadratic.weights(1:q))
  bp.coeff.dist(distrib.6raters,weights = quadratic.weights(1:q))

#' Dataset describing Fleiss' Benchmarking Scale
#' 
#' This dataset contains information describing Fleiss' scale for benchmarking chance-corrected agreement 
#' coefficients such as Gwet AC1/AC2, Kappa and many others.
#' 
#' @format Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
#'  \describe{
#'     \item{lb.FL}{The interval lower bound}
#'     \item{ub.FL}{The interval upper bound}
#'     \item{interp.FL}{The interpretation}
#'  }   
#'  
#' @source Fleiss, J. L. (1981). Statistical Methods for Rates and Proportions. John Wiley & Sons.
"fleiss"

#' This dataset contains raw categorical ratings that 4 raters assigned to
#' 12 subjects.
#' 
#' Dataset of raw ratings assigned to 12 units by 4 raters. Each row is 
#' associated with a unit identifier and each column to a rater. The columns
#' are named Rater1, Rater2, Rater3 and Rater4 and contain the categories into
#' which the unit was assigned. Category values are 1, 2, 3, 4, 5 and some 
#' ratings are missing.
#' 
#' @format A data frame of 12 rows and 5 columns containing integer values 
#' 1, 2, 3, 4 and 5. Missing data points are represented with a dot (".").
#' integers.
#'  \describe{
#'     \item{Units}{Patient's identifier}
#'     \item{Rater1}{Category into which Rater1 classified the unit}
#'     \item{Rater2}{Category into which Rater2 classified the unit}
#'     \item{Rater3}{Category into which Rater3 classified the unit}
#'     \item{Rater4}{Category into which Rater4 classified the unit}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"x.raw12x4"

#' Function for computing the Identity Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of identity weights to be used for calculating the unweighted coefficients.
#' @export
identity.weights<-function(categ){
	weights<-diag(length(categ))
	return (weights)
}

#' Function for computing the Quadratic Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
quadratic.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      weights[k,l] <- 1-(categ.vec[k]-categ.vec[l])^2/(xmax-xmin)^2 
	    }
	  }
	}
	return (weights)
}
#' Function for computing the Linear Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
linear.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      weights[k,l] <- 1-abs(categ.vec[k]-categ.vec[l])/abs(xmax-xmin)
	    }
	  }
	}
	return (weights)
}
#--------------------------------
#' Function for computing the Radical Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
radical.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      weights[k,l] <- 1-sqrt(abs(categ.vec[k]-categ.vec[l]))/sqrt(abs(xmax-xmin))
	    }
	  }
	}
	return (weights)
}

#--------------------------------
#' Function for computing the Ratio Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
ratio.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      weights[k,l] <- 1-((categ.vec[k]-categ.vec[l])/(categ.vec[k]+categ.vec[l]))^2 / ((xmax-xmin)/(xmax+xmin))^2
	    }
	  }
	}
	return (weights)
}

#--------------------------------
#' Function for computing the Circular Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
circular.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	U = xmax-xmin+1
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      weights[k,l] <- (sin(pi*(categ.vec[k]-categ.vec[l])/U))^2
	    }
	  }
	  weights <- 1-weights/max(weights)
	}
	return (weights)
}

#--------------------------------
#' Function for computing the Bipolar Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
bipolar.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
	if (is.numeric(categ)) { 
	   categ.vec <- sort(categ)
	}
	else {
	   categ.vec<-1:length(categ)
	}
	xmin<-min(categ.vec)
	xmax<-max(categ.vec)
	if (q==1) weights[1,1] <- 1
	else{
	  for(k in 1:q){
	    for(l in 1:q){
	      if (k!=l)
	        weights[k,l] <- (categ.vec[k]-categ.vec[l])^2 / (((categ.vec[k]+categ.vec[l])-2*xmin)*(2*xmax-(categ.vec[k]+categ.vec[l])))
	      else weights[k,l] <- 0
	    }
	  }
	  weights <- 1-weights/max(weights)
	}
	return (weights)
}


#--------------------------------
#' Function for computing the Ordinal Weights
#' @param  categ A mandatory parameter representing the vector of all possible ratings.
#' @return A square matrix of quadratic weights to be used for calculating the weighted coefficients.
#' @export
ordinal.weights<-function(categ){
	q<-length(categ)
	weights <- diag(q)
  categ.vec<-1:length(categ)
  if (q==1) weights[1,1] <- 1
  else{
    for(k in 1:q){
      for(l in 1:q){
        nkl <- max(k,l)-min(k,l)+1
        weights[k,l] <- nkl * (nkl-1)/2
      }
    }
    weights <- 1-weights/max(weights)
  }
	return (weights)
}

#' Distribution of 6 psychiatrists by Subject/patient and diagnosis Category.
#' 
#' This dataset summarizes the ratings assigned by 6 psychiatrists classifying 15 patients into one of five categories
#' named "Depression", "Personal Disorder", "Schizophrenia", "Neurosis" and "Other".  
#' 
#' @format This dataset has 15 rows (for the 15 subjects) and 7 columns. Only the last 6 columns representing the
#' categories into which subjects are classified are used in the calculations.    
#'  \describe{
#'     \item{Subject}{This variable repsents the subject number.}
#'     \item{Personality.Disorder}{Personality disorder category}
#'     \item{Schizophrenia}{Schizophrenia Category}
#'     \item{Neurosis}{Neurosis category}
#'     \item{Other}{"Other" category}
#'  }   
#'  
#' @source Fleiss, J. L. (1971). Measuring nominal scale agreement among many raters, \emph{Psychological Bulletin}, 76, 
#' 378-382.
"distrib.6raters"

#' Dataset of categorical ratings assigned to 10 subjects and presented in the
#' form of a distribution of 4 raters by subject and category
#' 
#' This dataset shows how 4 raters classified 10 subjects into 5 categories
#' labeled as q1, q2, q3, q4 and q5. Each record is associated with a subject
#' and shows how the 4 raters are distributed across the 5 categories they
#' assigned the subject to.
#' 
#' @format A data frame of 10 rows and 6 columns.
#' integers.
#'  \describe{
#'     \item{subject}{Patient's identifier}
#'     \item{q1}{Number of raters who classified the subject into category q1}
#'     \item{q2}{Number of raters who classified the subject into category q2}
#'     \item{q3}{Number of raters who classified the subject into category q3}
#'     \item{q4}{Number of raters who classified the subject into category q4}
#'     \item{q5}{Number of raters who classified the subject into category q5}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"x.dist10x5"

#' Dataset describing the Altman's Benchmarking Scale
#' 
#' This dataset contains information describing the Altman scale for benchmarking chance-corrected agreement 
#' coefficients such as Gwet AC1/AC2, Kappa and many others.
#' 
#' @format Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
#'  \describe{
#'     \item{lb.AL}{The interval lower bound}
#'     \item{ub.AL}{The interval upper bound}
#'     \item{interp.AL}{The interpretation}
#'  }   
#'  
#' @source Altman, D.G. (1991). \emph{Practical Statistics for Medical Research}. Chapman and Hall.
#' 
"altman"

#' Ratings of 15 subjects from 4 raters  
#' 
#' This dataset contains raw ratings of 15 subjects produced by 4 raters named
#' as RaterA, RaterB, RaterC and RaterD. 
#' 
#' @format All 5 columns are of alphabetic type.
#'  \describe{
#'     \item{Gender}{Contains the subject's gender (M or F)}
#'     \item{RaterA}{Contains one of ratings a,b,c produced by RaterA}
#'     \item{RaterB}{Contains one of ratings a,b,c produced by RaterB}
#'     \item{RaterC}{Contains one of ratings a,b,c produced by RaterC}
#'     \item{RaterD}{Contains one of ratings a,b,c produced by RaterD}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"agreeCAC"

#' Distribution of 4 raters by subject and by category, for 14 Subjects that belong to 2 groups "G1" and "G2"
#' 
#' This dataset contains rating data in the form of a subject-level distribution of 4 raters by category the subject
#' was classified into. A total of 4 raters had to classify 14 subjects into one of 5 categories "a", "b", "c", "d", and "e". 
#' This dataset is different version of the more detailed "cac.raw.g1g2" dataset. While "cac.raw.g1g2" tells you
#' about the exact category into which each rater classified all subjects, "cac.dist.g1g2" on the other hand, can only tell 
#' you how many raters classified a given subject into a particular category.
#' @format This dataset contains ratings obtained from an experiment where 4 raters classified 14 subjects into 5 
#' possible categories labeled as a, b, c, d, and e. None of the 4 raters scored all 14 units. Therefore, 
#' some missing ratings appear in each of the columns associated with the 4 raters.
#' 
#' Note that only the the 4 last columns are to be used with the functions included in this package.  The first 2
#' columns only play a descriptive role and are not used in any calculation.
#'  \describe{
#'     \item{Group}{This variable represents the group name.}
#'     \item{Units}{This variable represents the unit number.}
#'     \item{a}{Number of raters who classified the subject represented by the row into category "a"}
#'     \item{b}{Number of raters who classified the subject represented by the row into category "b"}
#'     \item{c}{Number of raters who classified the subject represented by the row into category "c"}
#'     \item{d}{Number of raters who classified the subject represented by the row into category "d"}
#'     \item{e}{Number of raters who classified the subject represented by the row into category "e"}
#'  }   
"cac.dist.g1g2"

#' Dataset showing the distribution of 6 raters by psychiatric condition
#' 
#' This dataset contains rating data about 15 subjects or patients whose 
#' psychiatric condition was evaluated by 6 raters. Each column represents one 
#' condition and contains the number of raters who diagnosed the patient with
#' that condition.
#' 
#' @format The first column is of character type whereas the remaining columns 
#' contains integers.
#'  \describe{
#'     \item{Gender}{Contains subject's gender (M or F)}
#'     \item{Depression}{The interval upper bound}
#'     \item{Pers.Disorder}{Contains the number of raters who labeled the subject as having personal disorder}
#'     \item{Schizophrenia}{Contains the number of raters who labeled the subject as having schizophrenia}
#'     \item{Neurosis}{Contains the number of raters who labeled the subject as having Neurosis}
#'     \item{Other}{Contains the number of raters who labeled the subject as having Other psychiatric condition}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"agree.cac3rd"

#' Rating Data from 4 Raters and 12 Subjects.
#' 
#' This dataset contains data from a reliability experiment where 5 observers scored 15 units on a 4-point numeric scale 
#' based on the values 0, 1, 2 and 3.  
#' 
#' @format This dataset contains ratings obtained from an experiment where 4 raters classified 12 subjects into 5 
#' possible categories labeled as 1, 2, 3, 4, and 5. None of the 4 raters scored all 12 units. Therefore, 
#' some missing ratings in the form of "NA" appear in each of the columns associated with the 4 raters.
#' 
#' Note that only the the 4 last columns are to be used with the functions included in this package.  The first 
#' column only plays a descriptive role and is not used in any calculation.
#'  \describe{
#'     \item{Units}{This variable repsents the unit number.}
#'     \item{Rater1}{All ratings from rater 1}
#'     \item{Rater2}{All ratings from rater 2}
#'     \item{Rater3}{All ratings from rater 3}
#'     \item{Rater4}{All ratings from rater 4}
#'  }   
#'  
#' @source Gwet, K.L. (2014) \emph{Handbook of Inter-Rater Reliability}, 4th Edition, page #120. Advanced Analytics, LLC. 
"cac.raw4raters"

#' Distribution of 100 pregnant women by pregnancy type and by abstractor.
#' 
#' This dataset contains pregnancy type data collected from 100 women who entered an Emergency Room with a positive 
#' pregnancy test and a second condition, which is either abdominal pain or vaginal bleeding. After reviewing their 
#' medical records, 2 reviewers (also referred to as abstractors) classified them into one of the following three pregnancy
#' categories: Ectopic Pregnancy (Ectopic), Abnormal Intrauterine pregnancy (AIU) and Normal Intrauterine Pregnancy 
#' (NIU).
#' 
#' @format Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
#'  \describe{
#'     \item{Type}{Pregnancy Type.  This variable is shown here for information only and is never used by any function 
#'                 in the irrCAC package.}
#'     \item{Ectopic}{Ectopic Pregnancy}
#'     \item{AIU}{Abnormal Intrauterine Pregnancy}
#'     \item{NIU}{Normal Intrauterine Pregnancy}
#'  }   
#'  
#' @source Gwet, K.L. (2014). \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC.
"cont3x3abstractors"

#' Dataset of raw ratings from 4 Raters on 14 Subjects that belong to 2 groups named "G1" and "G2"
#' 
#' This dataset contains data from a reliability experiment where 4 raters identified as Rater1, Rater2, Rater3 and Rater4
#' scored 14 units on a 5-point alphabetical scale based on the values a, b, c, d and e. These 14 units are allocated to 
#' 2 groups named G1 and G2.  
#' 
#' @format This dataset contains ratings obtained from an experiment where 4 raters classified 14 subjects into 5 
#' possible categories labeled as a, b, c, d, and e. None of the 4 raters scored all 14 units. Therefore, 
#' some missing ratings appear in each of the columns associated with the 4 raters.
#' 
#' Note that only the the 4 last columns are to be used with the functions included in this package.  The first 2
#' columns only play a descriptive role and are not used in any calculation.
#'  \describe{
#'     \item{Group}{This variable repsents the unit number.}
#'     \item{Units}{This variable repsents the unit number.}
#'     \item{Rater1}{All ratings from rater 1}
#'     \item{Rater2}{All ratings from rater 2}
#'     \item{Rater3}{All ratings from rater 3}
#'     \item{Rater4}{All ratings from rater 4}
#'  }   
#'  
"cac.raw.g1g2"

#' Distribution of 4 raters by Category and Subject - Subjects allocated in 2 groups A and B.
#' 
#' This dataset summarizes the ratings assigned by 4 raters who classified 15 subjects into one of 3 categories
#' named "a", "b", and "c".  
#' 
#' @format This dataset has 15 rows (for the 15 subjects) and 4 columns. Only the last 3 columns representing the
#' categories into which subjects are classified are used in the calculations - unless the sub-group analysis is required.    
#'  \describe{
#'     \item{Group}{This variable repsents the subject number.}
#'     \item{a}{category a}
#'     \item{b}{Category b}
#'     \item{c}{Category c}
#'  }   
#'  
"cac.dist4cat"

#								  AGREE.COEFF3.RAW.R
#						 		    (August 31, 2021)
#Description: This script file contains a series of R functions for computing various agreement coefficients
#	      for multiple raters (2 or more) when the input data file is in the form of nxr matrix or data frame showing 
#             the actual ratings each rater (column) assigned to each subject (in row). That is n = number of subjects, and r = number of raters.
#             A typical table entry (i,g) represents the rating associated with subject i and rater g. 
#Author: Kilem L. Gwet, Ph.D. (gwet@agreestat.com)
#-----------------------------------------------------------------
#     EXAMPLES OF SIMPLE CALLS OF THE MAIN FUNCTIONS:
# > pa.coeff.raw(YourRatings)       # to obtain the percent agreement coefficient
# > gwet.ac1.raw(YourRatings)       # to obtain gwet's AC1 coefficient
# > fleiss.kappa.raw(YourRatings)   # to obtain fleiss' unweighted generalized kappa coefficient
# > krippen.alpha.raw(YourRatings)  # to obtain krippendorff's unweighted alpha coefficient
# > conger.kappa.raw(YourRatings)   # to obtain conger's unweighted generalized kappa coefficient
# > bp.coeff.raw(YourRatings)       # to obtain Brennan-Prediger unweighted coefficient
#
#===========================================================================================
#pa.coeff.raw: Percent agreement coefficient and its standard error for multiple raters when input 
#		   dataset is a nxr matrix of alphanumeric ratings from n subjects and r raters 
#-------------
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#======================================================================================
#' Percent agreement among multiple raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.  
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work. 
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful 
#' in case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
#' coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @importFrom stats pt qt
#' @return A data list containing 3 objects: (1) a one-row data frame containing the estimates, (2) the weight matrix 
#' used in the calculations, and (3) the categories used in the analysis.  The data frame of estimates contains the 
#' following variables "coeff.name" (coefficient name), "pa" (the percent agreement), "pe" (percent chance-agreement-always equals 0),
#' "coeff.val" (agreement coefficient = pa), coeff.se (the percent agreement standard error), "conf.int" (the percent agreement confidence interval),
#' "p.value"(the percent agreement p-value), "w.name"(the weights' identification). 
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' pa.coeff.raw(cac.raw4raters) #Percent agreement, precision measures, weights & categories
#' pa.coeff.raw(cac.raw4raters)$est #Yields percent agreement with precision measures
#' pa <- pa.coeff.raw(cac.raw4raters)$est$coeff.val #Yields percent agreement alone.
#' pa
#' pa.coeff.raw(cac.raw4raters, weights = "quadratic") #weighted percent agreement/quadratic weights
#' @export
pa.coeff.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  pa<-NA;pe<-NA;coeff.val<-NA;coeff.se<-NA;conf.int<-NA;p.value<-NA;w.name<-NA;weights.mat<-NA
  ratings.mat <- as.matrix(ratings) 
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==''] <- NA_character_
  }
  n <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n/N # finite population correction 
  
  # creating a vector containing all categories used by the raters
  
  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)

  # creating the weights matrix
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom weights"
    weights.mat= as.matrix(weights)
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  } 
  
  # creating the nxq agreement matrix representing the distribution of raters by subjects and category
  
  agree.mat <- matrix(0,nrow=n,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))
  
  # calculating percent agreement coefficient
  
  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more
  pe <- 0
  coeff.val <- pa
  
  if (q>=2){    
    # calculating variance, stderr & p-value of gwet's ac1 coefficient
    
    den.ivec <- ri.vec*(ri.vec-1)
    den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
    pa.ivec <- (n/n2more)*(sum.q/den.ivec)
    var.pa<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
    if (n>=2){
      var.pa <- ((1-f)/(n*(n-1))) * sum((pa.ivec - pa)^2)
      stderr <- sqrt(var.pa)# pa's standard error
      stderr.est <- round(stderr,5)
      p.value <- 1-pt(pa/stderr,n-1)
      lcb <- pa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
      ucb <- min(1,pa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound 
    }
    conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
    coeff.se <- stderr.est
  }
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  coeff.name <- "Percent Agreement"
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,
                       conf.int,p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}
#===========================================================================================
#gwet.ac1.raw: Gwet's AC1/Ac2 coefficient (Gwet(2008)) and its standard error for multiple raters when input 
#		   dataset is a nxr matrix of alphanumeric ratings from n subjects and r raters 
#-------------
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#Bibliography:
#Gwet, K. L. (2008). ``Computing inter-rater reliability and its variance in the presence of high
#		agreement." British Journal of Mathematical and Statistical Psychology, 61, 29-48.
#============================================================================================
#' Gwet's AC1/AC2 agreement coefficient among multiple raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful 
#' in case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
#' coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A data list containing 3 objects: (1) a one-row data frame containing various statistics including the 
#' requested agreement coefficient, (2) the weight matrix used in the calculations if any, and (3) the categories 
#' used in the analysis. These could be categories reported by the raters, or those that were available to the raters
#' whether they used them or not.  The output data frame contains the following variables: "coeff.name" 
#' (coefficient name), "pa" (the percent agreement), "pe" (the percent chance agreement), coeff.val (the agreement coefficient 
#' estimate-AC1 or AC2), "coeff.se" (the standard error), "conf.int" (AC1/AC2 confidence interval), "p.value"
#' (Gwet AC1/AC2 p-value), "w.name"(the weights' identification). 
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' gwet.ac1.raw(cac.raw4raters) #AC1 coefficient, precision measures, weights & categories
#' gwet.ac1.raw(cac.raw4raters)$est #Yields AC1 coefficient with precision measures
#' ac1 <- gwet.ac1.raw(cac.raw4raters)$est$coeff.val #Yields AC1 coefficient alone.
#' ac1
#' gwet.ac1.raw(cac.raw4raters, weights = "quadratic") #AC2 coefficient with quadratic wts
#' @references Gwet, K. L. (2008). ``Computing inter-rater reliability and its variance in the presence of high
#' agreement." \emph{British Journal of Mathematical and Statistical Psychology}, 61, 29-48.
#' @export
gwet.ac1.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  ratings.mat <- as.matrix(ratings)
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==''] <- NA_character_
  }
  n <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n/N # finite population correction 

  # creating a vector containing all categories used by the raters

  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)

  # creating the weights matrix

  if (is.character(weights)){
      w.name <- weights
      if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
      else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
      else if (weights=="linear") weights.mat<-linear.weights(categ)
      else if (weights=="radical") weights.mat<-radical.weights(categ)
      else if (weights=="ratio") weights.mat<-ratio.weights(categ)
      else if (weights=="circular") weights.mat<-circular.weights(categ)
      else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
      else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  }

  # creating the nxq agreement matrix representing the distribution of raters by subjects and category

  agree.mat <- matrix(0,nrow=n,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)    
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))
  
  # calculating gwet's ac1 coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more
  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  if (q>=2){pe <- sum(weights.mat) * sum(pi.vec*(1-pi.vec)) / (q*(q-1))}else pe=1e-15
  gwet.ac1 <- (pa-pe)/(1-pe)
  gwet.ac1.est <- round(gwet.ac1,5)
  
  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec

  pe.r2 <- pe*(ri.vec>=2)
  ac1.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  pe.ivec <- (sum(weights.mat)/(q*(q-1))) * (agree.mat%*%(1-pi.vec))/ri.vec
  ac1.ivec.x <- ac1.ivec - 2*(1-gwet.ac1) * (pe.ivec-pe)/(1-pe)
  var.ac1<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
  if (n>=2){
    var.ac1 <- ((1-f)/(n*(n-1))) * sum((ac1.ivec.x - gwet.ac1)^2)
    stderr <- sqrt(var.ac1)# ac1's standard error
    stderr.est <- round(stderr,5)
    p.value <- 1-pt(gwet.ac1/stderr,n-1)
    lcb <- gwet.ac1 - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,gwet.ac1 + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  if (q==1) coeff.name <- "AC1"
  else{
    if (sum(weights.mat)==q) coeff.name <- "AC1"
    else coeff.name <- "AC2"
  }
  coeff.val <- gwet.ac1.est
  coeff.se <- stderr.est
  #Note that dplyr::summarise does not accept data frames with duplicate columns.
  #Therefore, I save the original column names, assign different names and restore
  #original names after using the summarise function
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,conf.int,
                       p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}

#=====================================================================================
#fleiss.kappa.raw: This function computes Fleiss' generalized kappa coefficient (see Fleiss(1971)) and 
#		   its standard error for 3 raters or more when input dataset is a nxr matrix of alphanumeric 
#		   ratings from n subjects and r raters.
#-------------
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#Bibliography:
#Fleiss, J. L. (1981). Statistical Methods for Rates and Proportions. John Wiley & Sons.
#======================================================================================
#' Fleiss' generalized kappa among multiple raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful in 
#' case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
#' coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A data list containing 3 objects: (1) a one-row data frame containing various statistics including the 
#' requested agreement coefficient, (2) the weight matrix used in the calculations if any, and (3) the categories 
#' used in the analysis. These could be categories reported by the raters, or those that were available to the raters
#' whether they used them or not.  The output data frame contains the following variables: "coeff.name" 
#' (coefficient name-here it will be "Fleiss' Kappa"), "pa" (the percent agreement), "pe" (the percent chance agreement), coeff.val 
#' (the agreement coefficient estimate-Fleiss' Kappa), "coeff.se" (the standard error), "conf.int" (Fleiss Kappa's confidence 
#' interval), "p.value"(Fleiss Kappa's p-value), "w.name"(the weights' identification).
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' fleiss.kappa.raw(cac.raw4raters) #Fleiss' kappa, precision measures, weights & categories
#' fleiss.kappa.raw(cac.raw4raters)$est #Yields Fleiss' kappa with precision measures
#' fleiss <- fleiss.kappa.raw(cac.raw4raters)$est$coeff.val #Yields Fleiss' kappa alone.
#' fleiss
#' fleiss.kappa.raw(cac.raw4raters, weights = "quadratic") #weighted Fleiss' kappa/quadratic wts
#' @references Fleiss, J. L. (1981). \emph{Statistical Methods for Rates and Proportions}. John Wiley & Sons.
#' @export
fleiss.kappa.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  ratings.mat <- as.matrix(ratings) 
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==''] <- NA_character_
  }
  n <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n/N # finite population correction 

  # creating a vector containing all categories used by the raters
 
  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)

  # creating the weights matrix

  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio")weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  }
  
  # creating the nxq agreement matrix representing the distribution of raters by subjects and category

  agree.mat <- matrix(0,nrow=n,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating fleiss's generalized kappa coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  if (q>=2){pe <- sum(weights.mat * (pi.vec%*%t(pi.vec)))}else pe=1e-15
  fleiss.kappa <- (pa-pe)/(1-pe)
  fleiss.kappa.est <- round(fleiss.kappa,5)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec

  pe.r2 <- pe*(ri.vec>=2)
  kappa.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  pi.vec.wk. <- weights.mat%*%pi.vec
  pi.vec.w.k <- t(weights.mat)%*%pi.vec
  pi.vec.w <- (pi.vec.wk. + pi.vec.w.k)/2
  pe.ivec <- (agree.mat%*%pi.vec.w)/ri.vec
  kappa.ivec.x <- kappa.ivec - 2*(1-fleiss.kappa) * (pe.ivec-pe)/(1-pe)
  
  var.fleiss<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
  if (n>=2){
    var.fleiss <- ((1-f)/(n*(n-1))) * sum((kappa.ivec.x - fleiss.kappa)^2)
    stderr <- sqrt(var.fleiss)# kappa's standard error
    stderr.est <- round(stderr,5)
    p.value <- 1-pt(fleiss.kappa/stderr,n-1)
    lcb <- fleiss.kappa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,fleiss.kappa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Fleiss' Kappa"
  coeff.val <- fleiss.kappa.est
  coeff.se <- stderr.est
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,conf.int,
                       p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}
#=====================================================================================
#krippen.alpha.raw: This function computes Krippendorff's alpha coefficient (see Krippendorff(1970, 1980)) and 
#		   its standard error for 3 raters or more when input dataset is a nxr matrix of alphanumeric 
#		   ratings from n subjects and r raters.
#-------------
#The algorithm used to compute krippendorff's alpha is very different from anything that was published on this topic. Instead,
#it follows the equations presented by K. Gwet (2012)
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#Bibliography:
#Gwet, K. (2014). Handbook of Inter-Rater Reliability: The Definitive Guide to Measuring the Extent of Agreement Among 
#	Multiple Raters, 4th Edition.  Advanced Analytics, LLC;
#Krippendorff (1970). "Bivariate agreement coefficients for reliability of data." Sociological Methodology,2,139-150
#Krippendorff (1980). Content analysis: An introduction to its methodology (2nd ed.), New-bury Park, CA: Sage.
#======================================================================================
#' Krippendorff's alpha coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful in 
#' case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
#' coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A data list containing 3 objects: (1) a one-row data frame containing various statistics including the 
#' requested agreement coefficient-in this case, Krippendorff's alpha, (2) the weight matrix used in the calculations if any, and 
#' (3) the vector of categories used in the analysis. These could be categories reported by the raters, or those that were available 
#' to the raters whether they used them or not.  The output data frame contains the following variables: "coeff.name" (coefficient 
#' name), "pa" (the percent agreement), "pe" (the percent chance agreement), coeff.val (Krippendorff's alpha estimate), "coeff.se 
#' (standard error), conf.int" (Krippendorff alpha's confidence interval),"p.value" (Krippendorff alpha's p-value), "w.name" 
#' (the weights' identification).
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' krippen.alpha.raw(cac.raw4raters) #Alpha coeff. , precision measures, weights & categories
#' krippen.alpha.raw(cac.raw4raters)$est #Krippendorff's alpha with precision measures
#' alpha <- krippen.alpha.raw(cac.raw4raters)$est$coeff.val #Krippendorff's alpha alone.
#' alpha
#' krippen.alpha.raw(cac.raw4raters, weights = "quadratic") #weighted alpha/ quadratic wts
#' @references Gwet, K. (2014). \emph{Handbook of Inter-Rater Reliability: The Definitive Guide to Measuring the Extent of Agreement Among Multiple Raters}, 4th Edition.  Advanced Analytics, LLC.\cr\cr
#' Krippendorff (1970). ``Bivariate agreement coefficients for reliability of data." \emph{Sociological Methodology},2,139-150.\cr\cr
#' Krippendorff (1980). \emph{Content analysis: An introduction to its methodology} (2nd ed.), New-bury Park, CA: Sage.
#' @export
krippen.alpha.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  pa<-NA;pe<-NA;coeff.val<-NA;coeff.se<-NA;conf.int<-NA;p.value<-NA;w.name<-NA;weights.mat<-NA
  ratings.mat <- as.matrix(ratings) 
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==''] <- NA_character_
  }
  n0 <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n0/N # finite population correction 

  # creating a vector containing all categories used by the raters
 
  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)

  # creating the weights matrix
  
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat <- identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  } 
  
  # creating the nxq agreement matrix representing the distribution of raters by subjects and category
  
  agree.mat <- matrix(0,nrow=n0,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))
  
  # calculating krippendorff's alpha coefficient
  
  ri.vec <- agree.mat%*%rep(1,q)
  agree.mat <- agree.mat[(ri.vec>=2),]
  agree.mat.w <- agree.mat.w[(ri.vec>=2),]
  ri.vec <- ri.vec[(ri.vec>=2)]
  ri.mean <- mean(ri.vec)
  n <- nrow(as.matrix(agree.mat))
  epsi <- 1/sum(ri.vec)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%as.matrix(rep(1,q))
  paprime <-sum(sum.q/(ri.mean*(ri.vec-1)))/n
  pa <- (1-epsi)*paprime + epsi
  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/ri.mean))
  if (q>=2){pe <- sum(weights.mat * (pi.vec%*%t(pi.vec)))}else  pe=1e-15
  krippen.alpha <- (pa-pe)/(1-pe)
  krippen.alpha.est <- round(krippen.alpha,5)
  krippen.alpha.prime <- (paprime-pe)/(1-pe)
    
  if (q>=2){
    # calculating variance, stderr & p-value of krippendorff's alpha coefficient
    pa.ivec <- sum.q/(ri.mean*(ri.vec-1)) - paprime*(ri.vec-ri.mean)/ri.mean
    krippen.ivec <- (pa.ivec-pe)/(1-pe)
    pi.vec.wk. <- weights.mat%*%pi.vec
    pi.vec.w.k <- t(weights.mat)%*%pi.vec
    pi.vec.w <- (pi.vec.wk. + pi.vec.w.k)/2
    pe.ivec <- (agree.mat%*%pi.vec.w)/ri.mean - pe * (ri.vec-ri.mean)/ri.mean
    krippen.ivec.x <- krippen.ivec - 2*(1-krippen.alpha.prime) * (pe.ivec-pe)/(1-pe)
    var.krippen<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
    if (n>=2){
      var.krippen <- ((1-f)/(n*(n-1))) * sum((krippen.ivec.x - krippen.alpha.prime)^2)
      stderr <- sqrt(var.krippen)# alpha's standard error
      stderr.est <- round(stderr,5)
      p.value <- 1-pt(krippen.alpha/stderr,n0-1)
      lcb <- krippen.alpha - stderr*qt(1-(1-conflev)/2,n0-1) # lower confidence bound
      ucb <- min(1,krippen.alpha + stderr*qt(1-(1-conflev)/2,n0-1)) # upper confidence bound
    }
    conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
    coeff.se <- stderr.est
  }
  coeff.val <- krippen.alpha.est
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  coeff.name <- "Krippendorff's Alpha"
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,
                       conf.int,p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}
#===========================================================================================
#conger.kappa.raw: Conger's kappa coefficient (see Conger(1980)) and its standard error for multiple raters when input 
#		   dataset is a nxr matrix of alphanumeric ratings from n subjects and r raters 
#-------------
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#Bibliography:
#Conger, A. J. (1980), ``Integration and Generalization of Kappas for Multiple Raters,"
#		Psychological Bulletin, 88, 322-328.
#======================================================================================
#' Conger's generalized kappa coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful in 
#' case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
#' coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A data list containing 3 objects: (1) a one-row data frame containing various statistics including the requested agreement
#' coefficient, (2) the weight matrix used in the calculations if any, and (3) A vector of categories used in the analysis. These 
#' could be categories reported by the raters, or those available to the raters whether they used them or not.  The output data frame
#' contains the following variables: "coeff.name" (coefficient name), "pa" (the percent agreement), "pe" (the percent chance 
#' agreement), coeff.val (Conger's Kappa estimate), "coeff.se" (standard error), "conf.int" (Conger Kappa's confidence 
#' interval), "p.value"(agreement coefficient's p-value), "w.name"(the weights' identification).
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' conger.kappa.raw(cac.raw4raters) #Conger's kappa, precision stats, weights & categories
#' conger.kappa.raw(cac.raw4raters)$est #Conger's kappa with precision measures
#' conger <- conger.kappa.raw(cac.raw4raters)$est$coeff.val #Yields Conger's kappa alone.
#' conger
#' conger.kappa.raw(cac.raw4raters, weights = "quadratic") #weighted Conger's kappa
#' @references Conger, A. J. (1980), ``Integration and Generalization of Kappas for Multiple Raters," \emph{Psychological Bulletin}, 88, 322-328.
#' @export
conger.kappa.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  ratings.mat <- as.matrix(ratings) 
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==''] <- NA_character_
  }
  n <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n/N # finite population correction 

  # creating a vector containing all categories used by the raters
 
  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)
  # creating the weights matrix

  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat <- identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat <- weights
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  } 
  
  # creating the nxq agreement matrix representing the distribution of raters by subjects and category

  agree.mat <- matrix(0,nrow=n,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))
  # creating the rxq rater-category matrix representing the distribution of subjects by rater and category

  classif.mat <- matrix(0,nrow=r,ncol=q)
  for(k in 1:q){
  	with.mis <-(t(ratings.mat)==categ[k])
  	without.mis <- replace(with.mis,is.na(with.mis),FALSE)
  	classif.mat[,k] <- without.mis%*%rep(1,n)
  }
  # calculating conger's kappa coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more
  ng.vec <- classif.mat%*%rep(1,q)
  pgk.mat <- classif.mat/(ng.vec%*%rep(1,q))
  p.mean.k <- (t(pgk.mat)%*%rep(1,r))/r 
  s2kl.mat <- (t(pgk.mat)%*%pgk.mat - r * p.mean.k%*%t(p.mean.k))/(r-1)
  if (q>=2){pe <- sum(weights.mat * (p.mean.k%*%t(p.mean.k) -  s2kl.mat/r))}else  pe=1e-15
  conger.kappa <- (pa-pe)/(1-pe)
  conger.kappa.est <- round(conger.kappa,5)

  # calculating variance, stderr & p-value of conger's kappa coefficient
  
  bkl.mat <- (weights.mat+t(weights.mat))/2
  pe.ivec1 <- r*(agree.mat%*%t(t(p.mean.k)%*%bkl.mat))
  pe.ivec2 = rep(0,n)
  
  lamda.ig.mat=matrix(0,n,r)
  if (is.numeric(ratings.mat)){
  	epsi.ig.mat <-1-is.na(ratings.mat)
  	epsi.ig.mat <- replace(epsi.ig.mat,is.na(epsi.ig.mat),FALSE)
  }else{
  	epsi.ig.mat <- 1-(ratings.mat=="")
  	epsi.ig.mat <- replace(epsi.ig.mat,is.na(epsi.ig.mat),FALSE)
  }
  for(k in 1:q){
  	lamda.ig.kmat=matrix(0,n,r)
  	for(l in 1:q){
  		delta.ig.mat <- (ratings.mat==categ[l])
  		delta.ig.mat <- replace(delta.ig.mat,is.na(delta.ig.mat),FALSE)
  		lamda.ig.kmat <- lamda.ig.kmat + weights.mat[k,l] * (delta.ig.mat - (epsi.ig.mat - rep(1,n)%*%t(ng.vec/n)) * (rep(1,n)%*%t(pgk.mat[,l])))
  	}
  	lamda.ig.kmat = lamda.ig.kmat*(rep(1,n)%*%t(n/ng.vec))
  	lamda.ig.mat = lamda.ig.mat+ lamda.ig.kmat*(r*mean(pgk.mat[,k]) - rep(1,n)%*%t(pgk.mat[,k]))
  }
  pe.ivec <- (lamda.ig.mat%*%rep(1,r)) / (r*(r-1))
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec
  pe.r2 <- pe*(ri.vec>=2)
  conger.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe) 
  conger.ivec.x <- conger.ivec - 2*(1-conger.kappa) * (pe.ivec-pe)/(1-pe)
  
  var.conger<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
  if (n>=2){
    var.conger <- ((1-f)/(n*(n-1))) * sum((conger.ivec.x - conger.kappa)^2)
    stderr <- sqrt(var.conger)# conger's kappa standard error
    stderr.est <- round(stderr,5)
    p.value <- 1-pt(conger.kappa/stderr,n-1)
    lcb <- conger.kappa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,conger.kappa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Conger's Kappa"
  coeff.val <- conger.kappa.est
  coeff.se <- stderr.est
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,conf.int,
                       p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}
#===========================================================================================
#bp.coeff.raw: Brennan-Prediger coefficient (see Brennan & Prediger(1981)) and its standard error for multiple raters when input 
#		   dataset is a nxr matrix of alphanumeric ratings from n subjects and r raters 
#-------------
#The input data "ratings" is a nxr data frame of raw alphanumeric ratings
#from n subjects and r raters. Exclude all subjects that are not rated by any rater.
#Bibliography:
#Brennan, R.L., and Prediger, D. J. (1981). ``Coefficient Kappa: some uses, misuses, and alternatives."
#           Educational and Psychological Measurement, 41, 687-699.
#======================================================================================
#' Brennan & Prediger's (BP) agreement coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
#' @param ratings An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.
#' @param weights is a mandatory parameter that is either a string variable or a matrix. 
#' The string describes one of the predefined weights and must take one of the values 
#' ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number 
#' of posssible categories where a subject can be classified. If some of the q possible 
#' categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.
#' @param categ.labels An optional vector parameter containing the list of all possible ratings. It may be useful in case some of the
#' possibe ratings are not used by any rater, they will still be used when calculating agreement coefficients. The default value is 
#' NULL. In this case, only categories reported by the raters are used in the
#' calculations.
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value 
#' is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction
#' to the variance.  Its default value is infinity. 
#' @return A data list containing 3 objects: (1) a one-row data frame containing various statistics including the requested agreement
#' coefficient, (2) the weight matrix used in the calculations if any, and (3) A vector of categories used in the analysis. These 
#' could be categories reported by the raters, or those available to the raters whether they used them or not.  The output data frame
#' contains the following variables: "coeff.name" (coefficient name), "pa" (the percent agreement), "pe" (the percent chance 
#' agreement), coeff.val (Brennan-Prediger coefficient estimate), "coeff.se" (standard error), "conf.int" (the confidence interval), 
#' "p.value"(Brennan-Prediger coefficient's p-value), "w.name"(the weights' identification).
#' @examples 
#' #The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
#' cac.raw4raters
#' bp.coeff.raw(cac.raw4raters) #BP coefficient, precision measures, weights & categories
#' bp.coeff.raw(cac.raw4raters)$est #Brennan-Prediger coefficient with precision measures
#' bp <- bp.coeff.raw(cac.raw4raters)$est$coeff.val #Yields Brennan-Prediger coefficient alone.
#' bp
#' bp.coeff.raw(cac.raw4raters, weights = "quadratic") #weighted Brennan-Prediger coefficient
#' @references Brennan, R.L., & Prediger, D. J. (1981). ``Coefficient Kappa: some uses, misuses, and alternatives." \emph{Educational and Psychological Measurement}, 41, 687-699.
#' @export
bp.coeff.raw <- function(ratings,weights="unweighted",categ.labels=NULL,conflev=0.95,N=Inf){ 
  ratings.mat <- as.matrix(ratings) 
  if (is.character(ratings.mat)){
    ratings.mat <- trim(toupper(ratings.mat))
    ratings.mat[ratings.mat==""] <- NA_character_
  }
  n <- nrow(ratings.mat) # number of subjects
  r <- ncol(ratings.mat) # number of raters
  f <- n/N # finite population correction 

  # creating a vector containing all categories used by the raters
 
  if (is.null(categ.labels)){
    categ.init <- unique(na.omit(as.vector(ratings.mat)))
    categ <- sort(categ.init)
  }else categ <- toupper(categ.labels)
  q <- length(categ)

  # creating the weights matrix

  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
    if (sum(weights.mat)==q) w.name <- "Unweighted"
  } 
  
  # creating the nxq agreement matrix representing the distribution of raters by subjects and category

  agree.mat <- matrix(0,nrow=n,ncol=q)
  for(k in 1:q){
    categ.is.k <- (ratings.mat==categ[k])
    agree.mat[,k] <- (replace(categ.is.k,is.na(categ.is.k),FALSE)) %*% rep(1,r)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating gwet's ac1 coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  if (q>=2){pe <- sum(weights.mat) / (q^2)} else pe=1e-15
  bp.coeff <- (pa-pe)/(1-pe)
  bp.coeff.est <- round(bp.coeff,5)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec
  pe.r2 <- pe*(ri.vec>=2)
  bp.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  
  var.bp<-NA;stderr<-NA;stderr.est<-NA;p.value<-NA;lcb<-NA;ucb<-NA
  if (n>=2){
    var.bp <- ((1-f)/(n*(n-1))) * sum((bp.ivec - bp.coeff)^2)
    stderr <- sqrt(var.bp)# BP's standard error
    stderr.est <- round(stderr,5)
    p.value <- 1-pt(bp.coeff/stderr,n-1)
    lcb <- bp.coeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,bp.coeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Brennan-Prediger"
  coeff.val <- bp.coeff.est
  coeff.se <- stderr.est
  cnames <- colnames(ratings)
  colnames(ratings) <- sapply(1:r, function(x) paste0("dumcol", x))
  obs.count <- dplyr::summarise(as.data.frame(1-is.na(ratings)),across(1:r,sum))
  tot.obs <- sum((1-is.na(ratings)))
  colnames(obs.count) <- cnames
  df.out <- data.frame(coeff.name,pa,pe,coeff.val,coeff.se,conf.int,
                       p.value,tot.obs,w.name)
  return(list("est" = df.out,"weights" = weights.mat,
              "categories"=categ,"obs"=obs.count))
}
#
#-----  Additional functions needed to run the main functions. If the main functions must be included in another R script, then
# the user will need to add these additional functions to the new script file.
#
# ==============================================================
# trim(x): This is an r function for trimming leading and trealing blanks
# ==============================================================
#' An r function for trimming leading and trealing blanks
#' @param  x is a string variable.
#' @return A string variable where leading and trealing blanks are trimmed.
trim <- function( x ) {
  gsub("(^[[:space:]]+|[[:space:]]+$)", "", x) 
}
# # ==============================================================
# # The following functions generate various weight matrices used 
# # in the weighted or unweighted analyses.
# # ==============================================================
# identity.weights<-function(categ){
#   weights<-diag(length(categ))
#   return (weights)
# }
# #-----------------------------
# quadratic.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   for(k in 1:q){
#     for(l in 1:q){
#       weights[k,l] <- 1-(categ.vec[k]-categ.vec[l])^2/(xmax-xmin)^2 
#     }
#   }
#   return (weights)
# }
# linear.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   for(k in 1:q){
#     for(l in 1:q){
#       weights[k,l] <- 1-abs(categ.vec[k]-categ.vec[l])/abs(xmax-xmin)
#     }
#   }
#   return (weights)
# }
# #--------------------------------
# radical.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   for(k in 1:q){
#     for(l in 1:q){
#       weights[k,l] <- 1-sqrt(abs(categ.vec[k]-categ.vec[l]))/sqrt(abs(xmax-xmin))
#     }
#   }
#   return (weights)
# }
# #--------------------------------
# ratio.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   for(k in 1:q){
#     for(l in 1:q){
#       weights[k,l] <- 1-((categ.vec[k]-categ.vec[l])/(categ.vec[k]+categ.vec[l]))^2 / ((xmax-xmin)/(xmax+xmin))^2
#     }
#   }
#   return (weights)
# }
# 
# #--------------------------------
# circular.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   U = xmax-xmin+1
#   for(k in 1:q){
#     for(l in 1:q){
#       weights[k,l] <- (sin(pi*(categ.vec[k]-categ.vec[l])/U))^2
#     }
#   }
#   weights <- 1-weights/max(weights)
#   return (weights)
# }
# 
# #--------------------------------
# bipolar.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   if (is.numeric(categ)) { 
#     categ.vec <- sort(categ)
#   }
#   else {
#     categ.vec<-1:length(categ)
#   }
#   xmin<-min(categ.vec)
#   xmax<-max(categ.vec)
#   for(k in 1:q){
#     for(l in 1:q){
#       if (k!=l)
#         weights[k,l] <- (categ.vec[k]-categ.vec[l])^2 / (((categ.vec[k]+categ.vec[l])-2*xmin)*(2*xmax-(categ.vec[k]+categ.vec[l])))
#       else weights[k,l] <- 0
#     }
#   }
#   weights <- 1-weights/max(weights)
#   return (weights)
# }
# #--------------------------------
# ordinal.weights<-function(categ){
#   q<-length(categ)
#   weights <- diag(q)
#   categ.vec<-1:length(categ)
#   for(k in 1:q){
#     for(l in 1:q){
#       nkl <- max(k,l)-min(k,l)+1
#       weights[k,l] <- nkl * (nkl-1)/2
#     }
#   }
#   weights <- 1-weights/max(weights)
#   return (weights)
# }

#' Distribution of 223 Psychiatric Patients by Type of of Psychiatric Disorder and Diagnosis Method.
#' 
#' This dataset shows the distribution of 223 psychiatric patients by diagnosis category and by the method used to 
#' obtain the diagnosis. The first method named ``Clinical Diagnosis" (also known as ``Facility Diagnosis") is used 
#' in a service facility (e.g. public hospital, or a community unit) and does not rely on a rigorous application of 
#' research criteria. The second method known as ``Research Diagnosis" is based on a strict application of research 
#' criteria. Column 1 contains the diagnosis categories into which patients are classified with Method 1.  The first
#' row on the other hand, shows categories into which patients are classified with Method 2. 
#' 
#' @format This dataset contains a 4x4 squared table. The first column is never used in the calculations and only 
#' contains row names. Only the last 4 columns are used for computing agreement coefficients. 
#'  \describe{
#'     \item{Diagnosis}{Pregnancy Type. This variable is shown here for information only and is never used by any function 
#'                 in the irrCAC package.}
#'     \item{Schizophrenia}{Ectopic Pregnancy}
#'     \item{Bipolar.Disorder}{Abnormal Intrauterine Pregnancy}
#'     \item{Depression}{Normal Intrauterine Pregnancy}
#'     \item{Other}{Normal Intrauterine Pregnancy}
#'  }   
#'  
#' @source Gwet, K.L. (2014). \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC.
"cont4x4diagnosis"

#' Raw categorical ratings assigned to 10 subjects by 4 raters
#' 
#' Dataset of 10 subjects and the categorical ratings assigned to them by 4 
#' raters. The 5 categories available to raters are labeled as 1, 2, 3, 4, 5.
#' 
#' @format A data frame of 10 rows and 5 columns.
#' integers.
#'  \describe{
#'     \item{subject}{Patient's identifier}
#'     \item{rater1}{Category into which rater1 classified the subject}
#'     \item{rater2}{Category into which rater2 classified the subject}
#'     \item{rater3}{Category into which rater3 classified the subject}
#'     \item{rater4}{Category into which rater4 classified the subject}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"x.raw10x4"

#' Scores assigned by 5 observers to 20 experimental units.
#' 
#' This dataset contains data from a reliability experiment where 5 observers scored 15 units on a 4-point numeric scale 
#' based on the values 0, 1, 2 and 3.  
#' 
#' @format This dataset has 15 rows (for the 15 subjects) and 6 columns. Only the last 5 columns associated with the 5
#' observers are used in the calculations. Of the 5 observers, only observer 3 scored all 15 units. Therefore, some missing
#' ratings in the form of "NA" appear in the columns associated with the remaining 4 observers.
#'  \describe{
#'     \item{Unit}{This variable repsents the unit number.}
#'     \item{Observer1}{All ratings from Observer 1}
#'     \item{Observer2}{All ratings from Observer 2}
#'     \item{Observer3}{All ratings from Observer 3}
#'     \item{Observer4}{All ratings from Observer 4}
#'     \item{Observer5}{All ratings from Observer 5}
#'  }   
#'  
#' @source Gwet, K.L. (2014) \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC. 
#' \emph{A larger version of this table can be found on page #125}
"cac.raw5obser"

#' Distribution of 10 subjects by rater (Ben and Gerry) and by category.
#' 
#' This dataset contains rating data collected from 10 human subjects by 2 raters named
#' Ben and Gerry. While the 5 categories a, b, c, d, and e are available for rating, Gerry
#' never used category a.
#' 
#' @format Each row of this dataset describes a unique pair of categories and the number
#' of subjects that both Ben and Gerry classified into the 2 categories respectively.
#'  \describe{
#'     \item{Ben}{Categories that Ben used to classify the subjects.}
#'     \item{Gerry}{Categories that Gerry used to classify the subjects}
#'     \item{n}{Number of subjects that Ben and Gerry classified into the associated pair of categories.}
#'  }   
#'  
#' @source N/A.
"freqs.data"

#' Dataset of raw ratings by 2 raters and 12 subjects.
#' 
#' This dataset contains 12 rows and 2 columns. Each row represents a subject
#' and the 2 columns labeled as "rater1" and "rater2" contain the ratings 
#' produced by the 2 raters.  
#' 
#' @format This dataset contains ratings obtained from an experiment where 2 
#' raters classified 12 subjects into 3 categories A, B, and C. Two of the 12
#' subjects were rated by a single rater. Consequently, this dataset contains
#' 2 missing ratings that are identified with the symbol <NA>.
#' 
#'  \describe{
#'     \item{rater1}{All ratings from rater 1}
#'     \item{rater2}{All ratings from rater 2}
#'  }   
#'  
#' @source Gwet, K.L. (2021) \emph{Handbook of Inter-Rater Reliability: 
#' Volume 1}, 5th Edition. AgreeStat Analytics. 
"cac.raw2raters"

#' Dataset describing the Landis & Koch Benchmarking Scale
#' 
#' This dataset contains information describing the Landis & Koch scale for benchmarking chance-corrected agreement 
#' coefficients such as Gwet AC1/AC2, Kappa and many others.
#' 
#' @format Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
#'  \describe{
#'     \item{lb.LK}{The interval lower bound}
#'     \item{ub.LK}{The interval upper bound}
#'     \item{interp.LK}{The interpretation}
#'  }   
#'  
#' @source Landis, J.R. & Koch G. (1977). The measurement of observer agreement for categorical data, \emph{Biometrics}, 
#' 33, 159-174.
"landis.koch"

#					AGREE.COEFF3.DIST.R
#				 	 (July 29, 2019, Revised August 19, 2021)
#Description: This script file contains a series of R functions for computing various agreement coefficients
#		  for multiple raters (2 or more) when the input data file is in the form of nxq matrix or data frame showing 
#             the count of raters by subject and by category. That is n = number of subjects, and q = number of categories.
#             A typical table entry (i,k) represents the number of raters who classified subject i into category k. 
#Author: Kilem L. Gwet, Ph.D.
#

#===========================================================================================
#gwet.ac1.dist: Gwet's AC1/Ac2 coefficient (Gwet(2008)) and its standard error for multiple raters when input 
#		   dataset is a nxq matrix representing the distribution of raters by subject and by category. 
#-------------
#The input data "ratings" is an nxq matrix showing the number of raters by subject and category. A typical entry associated
#with a subject and a category, represents the number of raters who classified the subject into the specified category. Exclude 
#all subjects that are not rated by any rater.
#Bibliography:
#Gwet, K. L. (2008). ``Computing inter-rater reliability and its variance in the presence of high
#		agreement." British Journal of Mathematical and Statistical Psychology, 61, 29-48.
#============================================================================================
#' Gwet's AC1/AC2 agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
#' @param ratings An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.
#' @param weights is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
#' weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
#' can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parameter categ. Otherwise, only the categories reported will be used.
#' @param categ An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
#' some categories were not used by any rater inspite of being available to the raters. 
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @importFrom stats pt qt
#' @return A vector containing the following information: pa(the percent agreement),pe(the percent chance agreement),
#' coeff(Gwet's AC1 or AC2 dependending on whether weights are used or not),stderr(the standard error of Gwet's coefficient),
#' conf.int(the confidence interval of Gwet's coefficient), p.value(the p-value of Gwet's coefficient),coeff.name (AC1/AC2).
#' @examples 
#' #The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#' #by subject and by category. Note that each row of this dataset sums to the number of raters, which
#' #is 6. You may this dataset as follows:
#' distrib.6raters
#' gwet.ac1.dist(distrib.6raters) #AC1 coefficient, precision measures, weights & list of categories
#' ac1 <- gwet.ac1.dist(distrib.6raters)$coeff #Yields AC1 coefficient alone.
#' ac1
#' q <- ncol(distrib.6raters) #Number of categories
#' gwet.ac1.dist(distrib.6raters,weights = quadratic.weights(1:q)) #AC2 with quadratic weights
#' @source Gwet, K. L. (2008). ``Computing inter-rater reliability and its variance in the presence of high agreement," 
#' \emph{British Journal of Mathematical and Statistical Psychology}, 61, 29-48.
#' @importFrom dplyr across
#' @importFrom magrittr %>%
#' @export
gwet.ac1.dist <- function(ratings,weights="unweighted",categ=NULL,conflev=0.95,N=Inf){ 
  agree.mat <- as.matrix(ratings) 
  n <- nrow(agree.mat) # number of subjects
  q <- ncol(agree.mat) # number of categories
  f <- n/N # final population correction 

  # creating the weights matrix

  if (is.null(categ)) categ <- 1:q
    else{
      q2 <- length(categ)
      if (!is.numeric(categ)) categ <- 1:q2
      if (q2>q){
        colna1 <- colnames(agree.mat) 
        agree.mat <- cbind(agree.mat,matrix(0,n,(q2-q)))
        colna2 <- sapply(1:(q2-q),function(x) paste0("v",x))
        colnames(agree.mat) <- c(colna1,colna2)
        q <- q2
      } 
    }
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating gwet's ac1 coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  pe <- sum(weights.mat) * sum(pi.vec*(1-pi.vec)) / (q*(q-1))
  gwet.ac1 <- (pa-pe)/(1-pe)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec

  pe.r2 <- pe*(ri.vec>=2)
  ac1.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  pe.ivec <- (sum(weights.mat)/(q*(q-1))) * (agree.mat%*%(1-pi.vec))/ri.vec
  ac1.ivec.x <- ac1.ivec - 2*(1-gwet.ac1) * (pe.ivec-pe)/(1-pe)
  
  var.ac1 <- ((1-f)/(n*(n-1))) * sum((ac1.ivec.x - gwet.ac1)^2)
  stderr <- sqrt(var.ac1)# ac1's standard error
  p.value <- 1-pt(gwet.ac1/stderr,n-1)
  
  lcb <- gwet.ac1 - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
  ucb <- min(1,gwet.ac1 + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff <- gwet.ac1
  if (sum(weights.mat)==q) coeff.name <-"Gwet's AC1"
  else coeff.name <-"Gwet's AC2"
  
  r.min <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%min()
  r.max <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%max()
  df.out <- data.frame(coeff.name,coeff,stderr,conf.int,
                       p.value,pa,pe,n,q,r.min,r.max)  
  return(df.out)
}
#=====================================================================================
#fleiss.kappa.dist: This function computes Fleiss' generalized kappa coefficient (see Fleiss(1971)) and 
#		   its standard error for 3 raters or more when input dataset is a nxq matrix representing 
#              the distribution of raters by subject and by category. 
#-------------
#The input data "ratings" is an nxq matrix showing the number of raters by subject and category. A typical entry associated
#with a subject and a category, represents the number of raters who classified the subject into the specified category. Exclude 
#all subjects that are not rated by any rater.
#Bibliography:
#Fleiss, J. L. (1981). Statistical Methods for Rates and Proportions. John Wiley & Sons.
#============================================================================================
#' Fleiss' agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
#' @param ratings An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.
#' @param weights is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
#' weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
#' can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ. Otherwise, only the categories reported will be used.
#' @param categ An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
#' some categories were not used by any rater inspite of being available to the raters. 
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A vector containing the following information: 
#' pa(the percent agreement),pe(the percent chance agreement),coeff(Fleiss' agreement coefficient),
#' stderr(the standard error of Fleiss' coefficient),conf.int(the confidence interval of Fleiss Kappa coefficient),
#' p.value(the p-value of Fleiss' coefficient),coeff.name ("Fleiss").
#' @examples 
#' #The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#' #by subject and by category. Note that each row of this dataset sums to the number of raters, which
#' #is 6. You may this dataset as follows:
#' distrib.6raters
#' fleiss.kappa.dist(distrib.6raters) #Fleiss' kappa, precision measures, weights & list of categories
#' fleiss <- fleiss.kappa.dist(distrib.6raters)$coeff #Yields Fleiss' kappa alone.
#' fleiss
#' q <- ncol(distrib.6raters) #Number of categories
#' fleiss.kappa.dist(distrib.6raters,weights = quadratic.weights(1:q)) #Weighted fleiss/quadratic wts
#' @source Fleiss, J. L. (1981). \emph{Statistical Methods for Rates and Proportions}. John Wiley & Sons.
#' @export
fleiss.kappa.dist <- function(ratings,weights="unweighted",categ=NULL,conflev=0.95,N=Inf){ 
  agree.mat <- as.matrix(ratings) 
  n <- nrow(agree.mat) # number of subjects
  q <- ncol(agree.mat) # number of categories
  f <- n/N # final population correction 

  # creating the weights matrix
  
  if (is.null(categ)) categ <- 1:q
  else{
    q2 <- length(categ)
    if (!is.numeric(categ)) categ <- 1:q2
    if (q2>q){
      colna1 <- colnames(agree.mat) 
      agree.mat <- cbind(agree.mat,matrix(0,n,(q2-q)))
      colna2 <- sapply(1:(q2-q),function(x) paste0("v",x))
      colnames(agree.mat) <- c(colna1,colna2)
      q <- q2
    } 
  }
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
  }
  
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating fleiss's generalized kappa coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  pe <- sum(weights.mat * (pi.vec%*%t(pi.vec)))
  fleiss.kappa <- (pa-pe)/(1-pe)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec

  pe.r2 <- pe*(ri.vec>=2)
  kappa.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  pi.vec.wk. <- weights.mat%*%pi.vec
  pi.vec.w.k <- t(weights.mat)%*%pi.vec
  pi.vec.w <- (pi.vec.wk. + pi.vec.w.k)/2

  pe.ivec <- (agree.mat%*%pi.vec.w)/ri.vec
  kappa.ivec.x <- kappa.ivec - 2*(1-fleiss.kappa) * (pe.ivec-pe)/(1-pe)
  
  var.fleiss <- ((1-f)/(n*(n-1))) * sum((kappa.ivec.x - fleiss.kappa)^2)
  stderr <- sqrt(var.fleiss)# kappa's standard error
  p.value <- 1-pt(fleiss.kappa/stderr,n-1)
  
  lcb <- fleiss.kappa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
  ucb <- min(1,fleiss.kappa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff <- fleiss.kappa
  coeff.name <- "Fleiss' Kappa"
  r.min <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%min()
  r.max <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%max()
  df.out <- data.frame(coeff.name,coeff,stderr,conf.int,
                       p.value,pa,pe,n,q,r.min,r.max)
  return(df.out)
}
#=====================================================================================
#krippen.alpha.dist: This function computes Krippendorff's alpha coefficient (see Krippendorff(1970, 1980)) and 
#		   its standard error for 3 raters or more when input dataset is a nxq matrix representing 
#              the distribution of raters by subject and by category. 
#The input data "ratings" is an nxq matrix showing the number of raters by subject and category. A typical entry associated
#with a subject and a category, represents the number of raters who classified the subject into the specified category. Exclude 
#all subjects that are not rated by any rater.
#-------------
#The algorithm used to compute krippendorff's alpha is very different from anything that was published on this topic. Instead,
#it follows the equations presented by K. Gwet (2010)
#Bibliography:
#Gwet, K. (2012). Handbook of Inter-Rater Reliability: The Definitive Guide to Measuring the Extent of Agreement Among 
#	Multiple Raters, 3rd Edition.  Advanced Analytics, LLC; 3rd edition (March 2, 2012)
#Krippendorff (1970). "Bivariate agreement coefficients for reliability of data." Sociological Methodology,2,139-150
#Krippendorff (1980). Content analysis: An introduction to its methodology (2nd ed.), New-bury Park, CA: Sage.
#============================================================================================
#' Krippendorff's agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
#' @param ratings An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.
#' @param weights is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
#' weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
#' can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ. Otherwise, only the categories reported will be used.
#' @param categ An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
#' some categories were not used by any rater inspite of being available to the raters. 
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A vector containing the following information: 
#' pa(the percent agreement),pe(the percent chance agreement),coeff(Krippendorff's alpha),
#' stderr(the standard error of Krippendorff's coefficient),conf.int(the confidence interval of Krippendorff's alpha coefficient),
#' p.value(the p-value of Krippendorff's alpha), coeff.name ("krippen alpha").
#' @examples 
#' #The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#' #by subject and by category. Note that each row of this dataset sums to the number of raters, which
#' #is 6. You may this dataset as follows:
#' distrib.6raters
#' krippen.alpha.dist(distrib.6raters) #Krippendorff's alpha, precision measures, weights & categories
#' alpha <- krippen.alpha.dist(distrib.6raters)$coeff #Yields Krippendorff's alpha coefficient alone.
#' alpha
#' q <- ncol(distrib.6raters) #Number of categories
#' krippen.alpha.dist(distrib.6raters,weights = quadratic.weights(1:q)) #Weighted alpha
#' @source Gwet, K. (2014). \emph{Handbook of Inter-Rater Reliability: The Definitive Guide to Measuring the Extent of Agreement Among 
#' Multiple Raters}, 4th Edition.  Advanced Analytics, LLC
#' Krippendorff (1970). ``Bivariate agreement coefficients for reliability of data," \emph{Sociological Methodology},2,139-150
#' Krippendorff (1980). \emph{Content analysis: An introduction to its methodology (2nd ed.)}, New-bury Park, CA: Sage.
#' @export
krippen.alpha.dist <- function(ratings,weights="unweighted",categ=NULL,conflev=0.95,N=Inf){ 
  agree.mat <- as.matrix(ratings) 
  n <- nrow(agree.mat) # number of subjects
  q <- ncol(agree.mat) # number of categories
  f <- n/N # final population correction 

  # creating the weights matrix
  
  if (is.null(categ)) categ <- 1:q
  else{
    q2 <- length(categ)
    if (!is.numeric(categ)) categ <- 1:q2
    if (q2>q){
      colna1 <- colnames(agree.mat) 
      agree.mat <- cbind(agree.mat,matrix(0,n,(q2-q)))
      colna2 <- sapply(1:(q2-q),function(x) paste0("v",x))
      colnames(agree.mat) <- c(colna1,colna2)
      q <- q2
    } 
  }
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
  }
  
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating krippendorff's alpha coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  agree.mat<-agree.mat[(ri.vec>=2),]
  agree.mat.w <- agree.mat.w[(ri.vec>=2),]
  ri.vec <- ri.vec[(ri.vec>=2)]
  ri.mean <- mean(ri.vec)
  n <- nrow(agree.mat)
  epsi <- 1/sum(ri.vec)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  pa <- (1-epsi)* sum(sum.q/(ri.mean*(ri.vec-1)))/n + epsi

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/ri.mean))
  pe <- sum(weights.mat * (pi.vec%*%t(pi.vec)))
  krippen.alpha <- (pa-pe)/(1-pe)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.mean*(ri.vec-1)
  pa.ivec <- sum.q/den.ivec
  pa.v <- mean(pa.ivec)
  pa.ivec <- (1-epsi)*(pa.ivec-pa.v*(ri.vec-ri.mean)/ri.mean) + epsi

  krippen.ivec <- (pa.ivec-pe)/(1-pe)
  pi.vec.wk. <- weights.mat%*%pi.vec
  pi.vec.w.k <- t(weights.mat)%*%pi.vec

  pi.vec.w <- (pi.vec.wk. + pi.vec.w.k)/2

  pe.ivec <- (agree.mat%*%pi.vec.w)/ri.mean - sum(pi.vec) * (ri.vec-ri.mean)/ri.mean
  krippen.ivec.x <- krippen.ivec - (1-krippen.alpha) * (pe.ivec-pe)/(1-pe)
  
  var.krippen <- ((1-f)/(n*(n-1))) * sum((krippen.ivec.x - krippen.alpha)^2)
  stderr <- sqrt(var.krippen)# alpha's standard error
  p.value <- 1-pt(krippen.alpha/stderr,n-1)
  
  lcb <- krippen.alpha - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
  ucb <- min(1,krippen.alpha + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff <- krippen.alpha
  coeff.name <-"Krippendorff's Alpha"
  
  r.min <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%min()
  r.max <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%max()
  df.out <- data.frame(coeff.name,coeff,stderr,conf.int,
                       p.value,pa,pe,n,q,r.min,r.max)
  return(df.out)
}
#===========================================================================================
#bp.coeff.dist: Brennan-Prediger coefficient (see Brennan & Prediger(1981)) and its standard error for multiple raters when input 
#		   dataset is a nxq matrix representing the distribution of raters by subject and by category. 
#The input data "ratings" is an nxq matrix showing the number of raters by subject and category. A typical entry associated
#with a subject and a category, represents the number of raters who classified the subject into the specified category. Exclude 
#all subjects that are not rated by any rater.
#--------------------------------------------
#Bibliography:
#Brennan, R.L., and Prediger, D. J. (1981). ``Coefficient Kappa: some uses, misuses, and alternatives."
#           Educational and Psychological Measurement, 41, 687-699.
#======================================================================================
#' Brennan-Prediger's agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
#' @param ratings An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.
#' @param weights is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
#' weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
#' can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ. Otherwise, only the categories reported will be used.
#' @param categ An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
#' some categories were not used by any rater inspite of being available to the raters. 
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A vector containing the following information: 
#' pa(the percent agreement),pe(the percent chance agreement),coeff(Brennan-Prediger coefficient),
#' stderr(the standard error of Brennan-Prediger coefficient),conf.int(the p-value of Brennan-Prediger coefficient), 
#' p.value(the p-value of Brennan-Prediger coefficient),coeff.name ("Brennan-Prediger").
#' @examples 
#' #The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#' #by subject and by category. Note that each row of this dataset sums to the number of raters, which
#' #is 6. You may this dataset as follows:
#' distrib.6raters
#' bp.coeff.dist(distrib.6raters) #BP coefficient, precision measures, weights & list of categories
#' bp <- bp.coeff.dist(distrib.6raters)$coeff #Yields Brennan-Prediger coefficient alone.
#' bp
#' q <- ncol(distrib.6raters) #Number of categories
#' bp.coeff.dist(distrib.6raters,weights = quadratic.weights(1:q)) #Weighted BP with quadratic weights
#' @source Brennan, R.L., and Prediger, D. J. (1981). ``Coefficient Kappa: some uses, misuses, and alternatives," 
#' \emph{Educational and Psychological Measurement}, 41, 687-699.
#' @export
bp.coeff.dist <- function(ratings,weights="unweighted",categ=NULL,conflev=0.95,N=Inf){ 
  agree.mat <- as.matrix(ratings) 
  n <- nrow(agree.mat) # number of subjects
  q <- ncol(agree.mat) # number of categories
  f <- n/N # final population correction 

  # creating the weights matrix
  
  if (is.null(categ)) categ <- 1:q
  else{
    q2 <- length(categ)
    if (!is.numeric(categ)) categ <- 1:q2
    if (q2>q){
      colna1 <- colnames(agree.mat) 
      agree.mat <- cbind(agree.mat,matrix(0,n,(q2-q)))
      colna2 <- sapply(1:(q2-q),function(x) paste0("v",x))
      colnames(agree.mat) <- c(colna1,colna2)
      q <- q2
    } 
  }
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
  }
  
  agree.mat.w <- t(weights.mat%*%t(agree.mat))

  # calculating gwet's ac1 coefficient

  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more

  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  pe <- sum(weights.mat) / (q^2)
  bp.coeff <- (pa-pe)/(1-pe)

  # calculating variance, stderr & p-value of gwet's ac1 coefficient
  
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec

  pe.r2 <- pe*(ri.vec>=2)
  bp.ivec <- (n/n2more)*(pa.ivec-pe.r2)/(1-pe)
  var.bp <- ((1-f)/(n*(n-1))) * sum((bp.ivec - bp.coeff)^2)
  stderr <- sqrt(var.bp)# BP's standard error
  p.value <- 1-pt(bp.coeff/stderr,n-1)
  
  lcb <- bp.coeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
  ucb <- min(1,bp.coeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff <- bp.coeff
  coeff.name <- "Brennan-Prediger"
  
  r.min <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%min()
  r.max <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%max()
  df.out <- data.frame(coeff.name,coeff,stderr,conf.int,
                       p.value,pa,pe,n,q,r.min,r.max)
  return(df.out)
}
#======================================================================================
#' Percent agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
#' @param ratings An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.
#' @param weights is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
#' weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
#' If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
#' can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
#' possible categories as a vector in parametr categ. Otherwise, only the categories reported will be used.
#' @param categ An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
#' some categories were not used by any rater inspite of being available to the raters. 
#' @param conflev An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.
#' @param N An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity. 
#' @return A vector containing the following information: 
#' pa(the percent agreement),pe(the percent chance agreement),coeff(Brennan-Prediger coefficient),
#' stderr(the standard error of Brennan-Prediger coefficient),conf.int(the p-value of Brennan-Prediger coefficient), 
#' p.value(the p-value of Brennan-Prediger coefficient),coeff.name ("Brennan-Prediger").
#' @examples 
#' #The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#' #by subject and by category. Note that each row of this dataset sums to the number of raters, which
#' #is 6. You may this dataset as follows:
#' distrib.6raters
#' pa.coeff.dist(distrib.6raters) #percent agreement, precision measures, weights& list of categories
#' pa <- pa.coeff.dist(distrib.6raters)$coeff #Yields the percent agreement coefficient alone.
#' pa
#' q <- ncol(distrib.6raters) #Number of categories
#' pa.coeff.dist(distrib.6raters,weights = quadratic.weights(1:q)) #Weighted percent agreement
#' @source Brennan, R.L., and Prediger, D. J. (1981). ``Coefficient Kappa: some uses, misuses, and alternatives," 
#' \emph{Educational and Psychological Measurement}, 41, 687-699.
#' @export
pa.coeff.dist <- function(ratings,weights="unweighted",categ=NULL,conflev=0.95,N=Inf){ 
  agree.mat <- as.matrix(ratings) 
  n <- nrow(agree.mat) # number of subjects
  q <- ncol(agree.mat) # number of categories
  f <- n/N # final population correction 
  
  # creating the weights matrix
  
  if (is.null(categ)) categ <- 1:q
  else{
    q2 <- length(categ)
    if (!is.numeric(categ)) categ <- 1:q2
    if (q2>q){
      colna1 <- colnames(agree.mat) 
      agree.mat <- cbind(agree.mat,matrix(0,n,(q2-q)))
      colna2 <- sapply(1:(q2-q),function(x) paste0("v",x))
      colnames(agree.mat) <- c(colna1,colna2)
      q <- q2
    } 
  }
  if (is.character(weights)){
    w.name <- weights
    if (weights=="quadratic") weights.mat<-quadratic.weights(categ)
    else if (weights=="ordinal") weights.mat<-ordinal.weights(categ)
    else if (weights=="linear") weights.mat<-linear.weights(categ)
    else if (weights=="radical") weights.mat<-radical.weights(categ)
    else if (weights=="ratio") weights.mat<-ratio.weights(categ)
    else if (weights=="circular") weights.mat<-circular.weights(categ)
    else if (weights=="bipolar") weights.mat<-bipolar.weights(categ)
    else weights.mat<-identity.weights(categ)
  }else{
    w.name <- "Custom Weights"
    weights.mat= as.matrix(weights)
  }
  agree.mat.w <- t(weights.mat%*%t(agree.mat))
  # calculating percent agreement
  ri.vec <- agree.mat%*%rep(1,q)
  sum.q <- (agree.mat*(agree.mat.w-1))%*%rep(1,q)
  n2more <- sum(ri.vec>=2)
  pa <- sum(sum.q[ri.vec>=2]/((ri.vec*(ri.vec-1))[ri.vec>=2]))/n2more
  # calculating variance, stderr & p-value of the percent agreement
  den.ivec <- ri.vec*(ri.vec-1)
  den.ivec <- den.ivec - (den.ivec==0) # this operation replaces each 0 value with -1 to make the next ratio calculation always possible.
  pa.ivec <- sum.q/den.ivec
  
  pi.vec <- t(t(rep(1/n,n))%*%(agree.mat/(ri.vec%*%t(rep(1,q)))))
  pa.ivec <- (n/n2more)*pa.ivec
  var.pa <- ((1-f)/(n*(n-1))) * sum((pa.ivec - pa)^2)
  stderr <- sqrt(var.pa)# kappa's standard error
  p.value <- 1-pt(pa/stderr,n-1)
  
  lcb <- pa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
  ucb <- min(1,pa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  pe <-0
  coeff <- pa
  coeff.name <- "Percent agreement"
  r.min <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%min()
  r.max <- dplyr::reframe(as.data.frame(agree.mat),ri=rowSums(across(1:q)))%>%max()
  df.out <- data.frame(coeff.name,coeff,stderr,conf.int,
                       p.value,pa,pe,n,q,r.min,r.max)
  return(df.out)
}

#' Rating Data from 4 Raters and 15 human Subjects, 9 of whom are female and 6 males.
#' 
#' This dataset contains data from a reliability experiment where 4 raters scored 15 units on a 3-point alphabetic scale 
#' based on the values a, b, and c.  
#' 
#' @format This dataset contains ratings obtained from an experiment where 4 raters classiffied 15 subjects into 3 
#' possible categories labeled as a, b, and c.
#' 
#' Note that only the the 4 last columns are to be used with the functions included in this package.  The first 
#' column only plays a descriptive role and is not to be used in any calculation.
#'  \describe{
#'     \item{Group}{This variable repsents the unit number.}
#'     \item{RaterA}{All ratings from rater 1}
#'     \item{RaterB}{All ratings from rater 2}
#'     \item{RaterC}{All ratings from rater 3}
#'     \item{RaterD}{All ratings from rater 4}
#'  }   
#'  
"cac.raw.gender"

#--------------------------------
#' Computing Altman's Benchmark Scale Membership Probabilities
#' @param coeff A mandatory parameter representing the estimated value of an agreement coefficient.
#' @param se A mandatory parameter representing the agreement coefficient standard error. 
#' @param BenchDF An optional parameter that is a 3-column data frame containing the Altman's benchmark scale 
#'    information. The 3 columns are the interval lower bound, upper bound, and their interpretation. The default value 
#'    is a small file contained in the package and named \emph{altman.RData}, which describes the official Altman's 
#'    scale intervals and their interpretation. 
#' @importFrom stats na.omit pnorm
#' @return A one-column matrix containing the membership probabilities (c.f. \url{https://agreestat.com/papers/inter-rater\%20reliability\%20study\%20design1.pdf}) 
#' @export
altman.bf <- function(coeff,se,BenchDF=altman){
  n <- nrow(BenchDF)
  cmprob <-matrix(0,n,1)
  outprob <- matrix("",n,1)  
  truncate.fact <- pnorm((coeff+1)/se)-pnorm((coeff-1)/se)
  for (i in 1:n){
    if (i==1) {
      cmprob[i] = (pnorm((coeff-BenchDF$lb.AL[i])/se)-pnorm((coeff-BenchDF$ub.AL[i])/se))/truncate.fact
    }else{
      cmprob[i] = cmprob[i-1] + (pnorm((coeff-BenchDF$lb.AL[i])/se)-pnorm((coeff-BenchDF$ub.AL[i])/se))/truncate.fact
    }
    outprob[i] = round(cmprob[i],5)
  }
  al.out <- data.frame(altman$interp.AL,outprob)
  colnames(al.out) <- c("Altman","CumProb")
  rownames(al.out) <- paste0("(",altman$lb.AL," to ",altman$ub.AL,")")
  return(al.out)
}
#--------------------------------
#' Computing Landis-Koch Benchmark Scale Membership Probabilities 
#' @param coeff A mandatory parameter representing the estimated value of an agreement coefficient.
#' @param se A mandatory parameter representing the agreement coefficient standard error. 
#' @param BenchDF An optional parameter that is a 3-column data frame containing the Landis & Koch's benchmark scale 
#'    information. The 3 columns are the interval lower bound, upper bound, and their interpretation. The default value 
#'    is a small file contained in the package and named \emph{landis.koch.RData}, which describes the official 
#'    Landis & Koch's scale intervals and their interpretation.
#' @return A one-column matrix containing the membership probabilities (c.f. \url{https://agreestat.com/papers/inter-rater\%20reliability\%20study\%20design1.pdf}) 
#' @export
landis.koch.bf <- function(coeff,se,BenchDF=landis.koch){
  n <- nrow(BenchDF)
  cmprob <- matrix(0,n,1)
  outprob <- matrix("",n,1)
  truncate.fact <- pnorm((coeff+1)/se)-pnorm((coeff-1)/se)
  for (i in 1:n){
    if (i==1) {
      cmprob[i,1]=(pnorm((coeff-BenchDF$lb.LK[i])/se)-pnorm((coeff-BenchDF$ub.LK[i])/se))/truncate.fact
    }else{
      cmprob[i,1]=cmprob[i-1,1]+(pnorm((coeff-BenchDF$lb.LK[i])/se)-pnorm((coeff-BenchDF$ub.LK[i])/se))/truncate.fact
    }
    outprob[i]=round(cmprob[i],5)
  }
  lk.out <- data.frame(landis.koch$interp.LK,outprob)
  colnames(lk.out) <- c("Landis-Koch","CumProb")
  rownames(lk.out) <- paste0("(",landis.koch$lb.LK," to ",landis.koch$ub.LK,")")
  return(lk.out)
}
#--------------------------------
#' Computing Fleiss Benchmark Scale Membership Probabilities
#' @param coeff A mandatory parameter representing the estimated value of an agreement coefficient.
#' @param se A mandatory parameter representing the agreement coefficient standard error. 
#' @param BenchDF An optional parameter that is a 3-column data frame containing the Fleiss' benchmark scale information. 
#'    The 3 columns are the interval lower bound, upper bound, and their interpretation. The default value is a small 
#'    file contained in the package and named \emph{fleiss.RData}, which describes the fleiss' scale intervales and their
#'    interpretation. 
#' @return A one-column matrix containing the membership probabilities (c.f. \url{https://agreestat.com/papers/inter-rater\%20reliability\%20study\%20design1.pdf}) 
#' @export
fleiss.bf <- function(coeff,se,BenchDF=fleiss){
  n <- nrow(BenchDF)
  cmprob <-matrix(0,3,1)
  outprob <- matrix("",3,1)  
  truncate.fact <- pnorm((coeff+1)/se)-pnorm((coeff-1)/se)
  for (i in 1:3){
    if (i==1) {
      cmprob[i] = (pnorm((coeff-BenchDF$lb.FL[i])/se)-pnorm((coeff-BenchDF$ub.FL[i])/se))/truncate.fact
    }else{
      cmprob[i] = cmprob[i-1] + (pnorm((coeff-BenchDF$lb.FL[i])/se)-pnorm((coeff-BenchDF$ub.FL[i])/se))/truncate.fact
    }
    outprob[i]=round(cmprob[i],5)
  }
  fl.out <- data.frame(fleiss$interp.FL,outprob)
  colnames(fl.out) <- c("Fleiss","CumProb")
  rownames(fl.out) <- paste0("(",fleiss$lb.FL," to ",fleiss$ub.FL,")")
  return(fl.out)
}

utils::globalVariables(c("altman", "fleiss", "landis.koch"))
#' @importFrom tidyr drop_na pivot_wider
#' @importFrom dplyr count mutate select all_of
#' @importFrom stringr str_trim
#' @importFrom tibble as_tibble
#' @importFrom stats setNames

#' Dataset showing how 6 psychiatrists classified 15 patients by their mental
#' health condition.
#' 
#' Dataset of 15 psychiatric patients where each row identifies a patient and
#' shows how the 6 psychiatrists distrubte across 5 mental conditions. These
#' 5 conditions are Depression, personality disorder, Schizophrenia, Neurosis,
#' and Other.
#' 
#' @format A data frame of 15 rows and 6 columns.
#' integers.
#'  \describe{
#'     \item{Subject}{Patient's identifier}
#'     \item{Depression}{Number of raters to have diagnosed the patient with depression}
#'     \item{Personality.Disorder}{Number of raters to have diagnosed the patient with personality disorder}
#'     \item{Schizophrenia}{Number of raters to have diagnosed the patient with schizophrenia}
#'     \item{Neurosis}{Number of raters to have diagnosed the patient with neurosis}
#'     \item{Other}{Number of raters to have diagnosed the patient with other condition}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"x.dist6x5psy"

#						  AGREE.COEFF2.R
#					     (August 25, 2019)
#Description: This script file contains a series of R functions for computing various agreement coefficients
#		  for 2 raters when the input data file is in the form of 2x2 contingency table showing the distribution
#             of subjects by rater, and by category.
#Author: Kilem L. Gwet, Ph.D.
#

#====================================================================================
#kappa2.table: Cohen's kappa (Cohen(1960)) coefficient and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Kappa coefficient for 2 raters
#' @param ratings A square or contingency table of ratings (assume no missing ratings). See the 2 datasets 
#' "cont3x3abstractors" and "cont4x4diagnosis" that come with this package as examples.
#' @param weights An optional matrix that contains the weights used in the weighted analysis.
#' @param conflev An optional confidence level for confidence intervals. The default value is the traditional 0.95.
#' @param N An optional population size.  The default value is infinity.
#' @importFrom stats pt qt
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' kappa2.table(cont3x3abstractors) #Yields Cohen's kappa along with precision measures
#' kappa <- kappa2.table(cont3x3abstractors)$coeff.val #Yields Cohen's kappa alone.
#' kappa
#' q <- nrow(cont3x3abstractors) #Number of categories
#' kappa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))#weighted kappa/quadratic wts
#' @export
kappa2.table <- function(ratings,weights = identity.weights(1:ncol(ratings)),conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
  	stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pa <- sum(weights * ratings/n) # percent agreement
  pk. <- (ratings%*%rep(1,q))/n
  p.l <- t((t(rep(1,q))%*%ratings)/n)
  pe <- sum(weights*(pk.%*%t(p.l)))
  kappa <- (pa - pe)/(1 - pe) # weighted kappa
  
  # 2 raters special case variance
  
  pkl <- ratings/n
  pb.k <- weights %*% p.l
  pbl. <- t(weights) %*% pk.
  sum1 <- 0
  for(k in 1:q){
    for(l in 1:q){
      sum1 <- sum1 + pkl[k,l]* (weights[k,l]-(1-kappa)*(pb.k[k] + pbl.[l]))^2
    }
  }
  var.kappa <- ((1-f)/(n*(1-pe)^2)) * (sum1 - (pa-2*(1-kappa)*pe)^2)
  var.kappa <- max(var.kappa,1e-100)
  stderr <- sqrt(var.kappa)# kappa standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(kappa/stderr,n-1)
    lcb <- kappa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,kappa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Cohen's Kappa"
  coeff.val <- kappa
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}

#scott2.table: Scott's pi coefficient (Scott(1955)) and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Scott's coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param weights An optional matrix that contains the weights used in the weighted analysis. By default, this parameter contaings the identity weight matrix, which leads to the unweighted analysis.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite population correction to the standard error. It's default value is infinity. 
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' scott2.table(cont3x3abstractors) #Yields Scott's Pi coefficient along with precision measures
#' scott <- scott2.table(cont3x3abstractors)$coeff.val #Yields Scott's coefficient alone.
#' scott
#' q <- nrow(cont3x3abstractors) #Number of categories
#' scott2.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #weighted Scott's coefficient
#' @export
scott2.table <- function(ratings,weights=identity.weights(1:ncol(ratings)),conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
  	stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pa <- sum(weights * ratings/n) # percent agreement

  pk. <- (ratings%*%rep(1,q))/n
  p.l <- t((t(rep(1,q))%*%ratings)/n)
  pi.k <- (pk.+p.l)/2
  pe <- sum(weights*(pi.k%*%t(pi.k)))
  scott <- (pa - pe)/(1 - pe) # weighted scott's pi coefficint

	# 2 raters special case variance

  pkl <- ratings/n	     #p_{kl}
  pb.k <- weights %*% p.l    #\ov{p}_{+k}
  pbl. <- t(weights) %*% pk. #\ov{p}_{l+}
  pbk  <- (pb.k + pbl.)/2    #\ov{p}_{k}
  sum1 <- 0
  for(k in 1:q){
  	for(l in 1:q){
  		sum1 <- sum1 + pkl[k,l] * (weights[k,l]-(1-scott)*(pbk[k] + pbk[l]))^2
  	}
  }
  var.scott <- ((1-f)/(n*(1-pe)^2)) * (sum1 - (pa-2*(1-scott)*pe)^2)
  var.scott <- max(var.scott,1e-100)
  stderr <- sqrt(var.scott)# Scott's standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(scott/stderr,n-1)
    lcb <- scott - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,scott + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  } 
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Scott's Pi"
  coeff.val <- scott
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}

#gwet.ac1.table: Gwet's AC1/Ac2 coefficient (Gwet(2008)) and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Gwet's AC1/AC2 coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param weights An optional matrix that contains the weights used in the weighted analysis. By default, this 
#' parameter contaings the identity weight matrix, which leads to the unweighted analysis.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence 
#' intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite
#' population correction to the standard error. It's default value is infinity.
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' gwet.ac1.table(cont3x3abstractors) #Yields AC1 along with precision measures
#' ac1 <- gwet.ac1.table(cont3x3abstractors)$coeff.val #Yields AC1 coefficient alone.
#' ac1
#' q <- nrow(cont3x3abstractors) #Number of categories
#' gwet.ac1.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #AC2 with quadratic weights
#' @export
gwet.ac1.table <- function(ratings,weights=identity.weights(1:ncol(ratings)),conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
  	stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pa <- sum(weights * ratings/n) # percent agreement

  pk. <- (ratings%*%rep(1,q))/n
  p.l <- t((t(rep(1,q))%*%ratings)/n)
  pi.k <- (pk.+p.l)/2
  tw <- sum(weights)
  pe <- tw * sum(pi.k *(1-pi.k))/(q*(q-1))
  gwet.ac1 <- (pa - pe)/(1 - pe) # gwet's ac1/ac2 coefficint

	# calculation of variance - standard error - confidence interval - p-value

  pkl <- ratings/n	     #p_{kl}
  sum1 <- 0
  for(k in 1:q){
  	for(l in 1:q){
  		sum1 <- sum1 + pkl[k,l] * (weights[k,l]-2*(1-gwet.ac1)*tw*(1-(pi.k[k] + pi.k[l])/2)/(q*(q-1)))^2
  	}
  }
  var.gwet <- ((1-f)/(n*(1-pe)^2)) * (sum1 - (pa-2*(1-gwet.ac1)*pe)^2)
  var.gwet <- max(var.gwet,1e-100)
  stderr <- sqrt(var.gwet)# ac1's standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(gwet.ac1/stderr,n-1)
    lcb <- gwet.ac1 - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,gwet.ac1 + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  if (sum(weights)==q) coeff.name <- "Gwet's AC1"
  else coeff.name <- "Gwet's AC2"
  coeff.val <- gwet.ac1
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}
#bp2.table: Brennan-Prediger coefficient (Brennan & Prediger (1981)) and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Brenann-Prediger coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param weights An optional matrix that contains the weights used in the weighted analysis. By default, this 
#' parameter contaings the identity weight matrix, which leads to the unweighted analysis.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence 
#' intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite
#' population correction to the standard error. It's default value is infinity.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' bp2.table(cont3x3abstractors) #Yields Brennan-Prediger's coefficient along with precision measures
#' bp <- bp2.table(cont3x3abstractors)$coeff.val #Yields Brennan-Prediger coefficient alone.
#' bp
#' q <- nrow(cont3x3abstractors) #Number of categories
#' bp2.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #Weighted BP coefficient
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @export
bp2.table <- function(ratings,weights=identity.weights(1:ncol(ratings)),conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
  	stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pa <- sum(weights * ratings/n) # percent agreement

  tw <- sum(weights)
  pe <- tw/(q^2)
  bp.coeff <- (pa - pe)/(1 - pe) # Brennan-Prediger coefficint

	# calculation of variance - standard error - confidence interval - p-value

  pkl <- ratings/n	     #p_{kl}
  sum1 <- 0
  for(k in 1:q){
  	for(l in 1:q){
  		sum1 <- sum1 + pkl[k,l] * weights[k,l]^2
  	}
  }
  var.bp <- ((1-f)/(n*(1-pe)^2)) * (sum1 - pa^2)
  var.bp <- max(var.bp,1e-100)
  stderr <- sqrt(var.bp)# bp's standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(bp.coeff/stderr,n-1)
    lcb <- bp.coeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,bp.coeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Brennan-Prediger"
  coeff.val <- bp.coeff
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}

#krippen2.table: Scott's pi coefficient (Scott(1955)) and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Krippendorff's Alpha coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param weights An optional matrix that contains the weights used in the weighted analysis. By default, this 
#' parameter contaings the identity weight matrix, which leads to the unweighted analysis.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence 
#' intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite
#' population correction to the standard error. It's default value is infinity.
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' krippen2.table(cont3x3abstractors) #Krippendorff's alpha along with precision measures
#' alpha <- krippen2.table(cont3x3abstractors)$coeff.val #Krippendorff's alpha alone.
#' alpha
#' q <- nrow(cont3x3abstractors) #Number of categories
#' krippen2.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #Weighted alpha coefficient
#' @export
krippen2.table <- function(ratings,weights=identity.weights(1:ncol(ratings)),conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
  	stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  epsi = 1/(2*n)
  pa0 <- sum(weights * ratings/n)
  pa <- (1-epsi)*pa0 + epsi # percent agreement

  pk. <- (ratings%*%rep(1,q))/n
  p.l <- t((t(rep(1,q))%*%ratings)/n)
  pi.k <- (pk.+p.l)/2
  pe <- sum(weights*(pi.k%*%t(pi.k)))
  kripp.coeff <- (pa - pe)/(1 - pe) # weighted Krippen's alpha coefficint

	# calculating variance

  pkl <- ratings/n	     #p_{kl}
  pb.k <- weights %*% p.l    #\ov{p}_{+k}
  pbl. <- t(weights) %*% pk. #\ov{p}_{l+}
  pbk  <- (pb.k + pbl.)/2    #\ov{p}_{k}
  kcoeff <- (pa0 - pe)/(1 - pe)
  sum1 <- 0
  for(k in 1:q){
	for(l in 1:q){
		sum1 <- sum1 + pkl[k,l] * (weights[k,l]-(1-kcoeff)*(pbk[k] + pbk[l]))^2
	}
  }
  var.kripp <- ((1-f)/(n*(1-pe)^2)) * (sum1 - (pa0-2*(1-kcoeff)*pe)^2)
  var.kripp <- max(var.kripp,1e-100)
  stderr <- sqrt(var.kripp)# Kripp. alpha's standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(kripp.coeff/stderr,n-1)
    lcb <- kripp.coeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,kripp.coeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Krippendorff's Alpha"
  coeff.val <- kripp.coeff
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}



#bangdiwala.table: Bangdiwala's B coefficient (see Shankar & Bangdiwala, 2014) and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Bangdiwala B coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite population correction to the standard error. It's default value is infinity. 
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' bangdiwala.table(cont3x3abstractors) #Yields Bangdiwala coefficient along with precision measures
#' Bcoeff <- bangdiwala.table(cont3x3abstractors)$coeff.val #Yields Bangdiwala's coefficient alone.
#' Bcoeff
#' q <- nrow(cont3x3abstractors) #Number of categories
#' @export
bangdiwala.table <- function(ratings,conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
    stop('The contingency table should have the same number of rows and columns!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pk. <- rowSums(ratings)/n
  p.k <- colSums(ratings)/n
  b1 <- sum((diag(ratings)/n)**2)
  b2 <- c(pk.%*%p.k)
  bcoeff <- as.vector(b1/b2) # Bangdiwala B coefficient

  # Variance of Bangdiwala's B coefficient
  
  pi.k <- (pk.+p.k)/2
  pkl = ratings/n
  pkk <- diag(ratings)/n
  var1 = 2*sum((pkk^2)*(pkk-2*bcoeff*pi.k))
  var2 = (bcoeff^2)*sum(pi.k*pk.*p.k + (pkl%*%pk.)*p.k)
  var = 2*(var1+var2)/(n*(b2^2))
  varB <- (1-f)*var
  varB <- max(varB,1e-100)
  stderr <- sqrt(varB)  #standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(bcoeff/stderr,n-1)
    lcb <- bcoeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,bcoeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  } 
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Bangdiwala's B"
  coeff.val <- bcoeff
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}


#bangdiwala2RR.fn: Bangdiwala's B coefficient (see Shankar & Bangdiwala, 2014) and 
#                  its standard error for 2 raters when input dataset is a made up of
#                  2 columns of raw data.
#======================================================================================
#' Bangdiwala B coefficient for 2 raters when input dataset is made up of 2 columns of raw data.
#' @param fra.ratings.raw A dataframe with 2 columns of raw ratings.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite population correction to the standard error. It's default value is infinity. 
#' @importFrom stats setNames
#' @importFrom tidyr drop_na pivot_wider
#' @importFrom dplyr count mutate select all_of
#' @importFrom stringr str_trim
#' @importFrom tibble as_tibble
#' @return A data frame containing the following 9 variables: coeff.name, b1, b2, 
#' coeff.val, coeff.se, conf.int, p.value, n and name of the weight used.
#' @examples 
#' #The dataset cac.ben.gerry comes with this package. Analyze it as follows:
#' bangdi <- bangdiwala2RR.fn(cac.ben.gerry[,c(3,4)]) #using only the last 2 columns.
#' #The result will be following:
#' c(bangdi$coeff.name,bangdi$pa,bangdi$pe,bangdi$coeff.val,bangdi$coeff.se,
#'   bangdi$conf.int,bangdi$p.val,bangdi$tot.obs)
#' #1 Bangdiwala''s B 0.1322314 0.2066116      0.64 0.2158518 (0.159,1) 7.083e-03      11
#' bangdi$w.name
#' #1 Identity
#' @export
bangdiwala2RR.fn <- function(fra.ratings.raw,conflev=0.95,N=Inf){
  # if (is.numeric(unlist(fra.ratings.raw))) {
  #   nums <- sapply(fra.ratings.raw, is.numeric)
  #   fra.ratings.raw[,nums] <- as.data.frame(apply(fra.ratings.raw[,nums],
  #                                                 2,as.character))
  # }
  tib.ratings <- fra.ratings.raw
  tib.ratings[tib.ratings==''] <- NA_character_
  fr.data <- tib.ratings %>%
    count(tib.ratings[1],tib.ratings[2]) %>%
    drop_na()
  ratings <- long2wide.fn(fr.data)
  n <- nrow(tib.ratings) # number of subjects
  if (n<=N) f <- n/N # finite population correction
  else f=0
  q <- ncol(ratings) # number of categories
  pk. <- rowSums(ratings)/n
  p.k <- colSums(ratings)/n
  b1 <- sum((diag(ratings)/n)**2)
  b2 <- c(pk.%*%p.k)
  bcoeff <- as.vector(b1/b2) # Bangdiwala B coefficient
  
  # Variance of Bangdiwala's B coefficient
  
  pi.k <- (pk.+p.k)/2
  pkl = ratings/n
  pkk <- diag(ratings)/n
  var1 = 2*sum((pkk^2)*(pkk-2*bcoeff*pi.k))
  var2 = (bcoeff^2)*sum(pi.k*pk.*p.k + (pkl%*%pk.)*p.k)
  var = 2*(var1+var2)/(n*(b2^2))
  varB <- (1-f)*var
  varB <- max(varB,1e-100)
  stderr <- sqrt(varB)  #standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(bcoeff/stderr,n-1)
    lcb <- bcoeff - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,bcoeff + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  } 
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Bangdiwala's B"
  pa <- b1
  pe <- b2
  coeff.val <- bcoeff
  coeff.se <- stderr
  tot.obs <- n
  w.name <- "Identity"
  #p.value <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,pa,pe,coeff.val,coeff.se,conf.int,p.value,tot.obs,w.name))
}


#pa2.table: Percent agreement coefficient and its standard error for 2 raters when input dataset is a contingency table
#-------------
#The input data "ratings" is a qxq contingency table showing the distribution of
#subjects by rater, when q is the number of categories.
#======================================================================================
#' Percent Agreement coefficient for 2 raters
#' @param ratings A square table of ratings (assume no missing ratings).
#' @param weights An optional matrix that contains the weights used in the weighted analysis. By default, this 
#' parameter contains the identity weight matrix, which leads to the unweighted analysis.
#' @param conflev An optional parameter that specifies the confidence level used for constructing confidence 
#' intervals. By default the function assumes the standard value of 95\%.
#' @param N An optional parameter representing the finite population size if any. It is used to perform the finite
#' population correction to the standard error. It's default value is infinity.
#' @return A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
#' @examples 
#' #The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
#' pa2.table(cont3x3abstractors) #Yields percent agreement along with precision measures
#' pa <- pa2.table(cont3x3abstractors)$coeff.val #Yields percent agreement alone.
#' pa
#' q <- nrow(cont3x3abstractors) #Number of categories
#' pa2.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #Weighted percent agreement
#' @export
pa2.table <- function(ratings,weights=identity.weights(1:ncol(ratings)),
                      conflev=0.95,N=Inf){
  ratings <- as.matrix(ratings)
  if(dim(ratings)[1] != dim(ratings)[2]){
    stop('The contingency table should have the same number of rows and columns!')
  }
  if (ncol(ratings) != ncol(weights)){
    stop('The weight matrix has fewer columns than the contingency table. Please revise your input weights!')
  }
  n <- sum(ratings) # number of subjects
  f <- n/N # finite population correction
  q <- ncol(ratings) # number of categories
  pa <- sum(weights * ratings/n) # percent agreement

  # calculation of variance - standard error - confidence interval - p-value

  pkl <- ratings/n	     #p_{kl}
  sum1 <- 0
  for(k in 1:q){
    for(l in 1:q){
      sum1 <- sum1 + pkl[k,l] * weights[k,l]^2
    }
  }
  var.pa <- ((1-f)/n) * (sum1 - pa^2)
  var.pa <- max(var.pa,1e-100)
  stderr <- sqrt(var.pa) # pa standard error
  p.value <- NA;lcb <- NA;ucb <- NA
  if (n>=2){
    p.value <- 1-pt(pa/stderr,n-1)
    lcb <- pa - stderr*qt(1-(1-conflev)/2,n-1) # lower confidence bound
    ucb <- min(1,pa + stderr*qt(1-(1-conflev)/2,n-1)) # upper confidence bound
  }
  conf.int <- paste0("(",round(lcb,3),",",round(ucb,3),")")
  coeff.name <- "Percent Agreement"
  coeff.val <- pa
  coeff.se <- stderr
  coeff.ci <- conf.int
  coeff.pval <- format(p.value,digits=4,nsmall=3,scientific = TRUE)
  return(data.frame(coeff.name,coeff.val,coeff.se,coeff.ci,coeff.pval))
}
#======================================================================================
#' long2wide.fn: This function transforms a 3-column dataset of frequencies to a square
#' matrix or a contingency table. This function uses the freq.supp.fn() function.
#' @param freqs.long A 3-column data frame, where the first 2 variables represent the categories
#' that both raters have actually used when classifying the subjects. The third and last 
#' variable is generally named "n" and represents the count of subjects that classified into
#' the 2 associated categories by both raters.
#' @importFrom tidyr drop_na pivot_wider
#' @importFrom dplyr count n mutate select all_of
#' @importFrom stringr str_trim
#' @importFrom tibble as_tibble
#' @importFrom stats setNames
#' @return A matrix that represents a contingency showing the distribution of subjects by
#' rater and category.
#' @examples 
#' #The dataset "freqs.data" comes with this package. Analyze it as follows:
#' long2wide.fn(freqs.data) #Yields a 5x5 matrix
#' #This will produce the following 5x5 matrix:
#' #> long2wide.fn(freqs.data)
#' #a b c d e
#' #a 0 1 0 1 0
#' #b 0 2 0 0 0
#' #c 0 0 3 0 0
#' #d 0 1 0 1 0
#' #e 0 0 0 0 1
#' @export
long2wide.fn <- function(freqs.long){
  if (is.character(as.matrix(freqs.long))) freqs.long[freqs.long==""] <- NA
  freq.tab <- freqs.long %>%
    drop_na()
  cat.vec <- as.vector(unique(na.omit(c(freqs.long[[1]],freqs.long[[2]]))))
  cat.vec <- str_trim(format(cat.vec)) #- convert cat.vec to a character vector if needed
  freq.tab1 <-freq.supp.fn(freq.tab,cat.vec)
  freq.tab2 <- as_tibble(freq.tab1[order(freq.tab1[[1]],freq.tab1[[2]]),])
  freq.tab2 <- mutate(freq.tab2,n=as.integer(freq.tab2[[3]]))
  dfra.mat <- pivot_wider(data=freq.tab2,names_from = colnames(freq.tab2)[2],
                          values_from = n,values_fill = 0)
  sfra.mat <- as.matrix(select(dfra.mat,all_of(cat.vec)))
  rownames(sfra.mat) <- colnames(sfra.mat)
  return(sfra.mat)
}
#=========================================================================================
#' freq.supp.fn: This function reads a 3-variable input data file containing unique pairs
#' of categories along with their frequency of occurrences, and outputs a similar file
#' where all possible pairs of categories are represented, some with a frequency of 
#' occurrence of 0. 
#' @param freq.data The input data file containing all unique combinations of reported 
#' categories.
#' @param categories.vec A vector containing the complete set of categories available to 
#' raters (e.g. "a", "b", "c", "d", "e"). The raters will not necessarily use all of 
#' these categories.
#' @return This function returns a complete data frame containing all possible combinations of
#' of categories in the categories.vec vector. Newly-added combinations of categories will
#' have a frequency occurrence of 0.
#' @examples 
#' #The dataset "freqs.data" comes with this package. Analyze it as follows:
#' freq.supp.fn(freqs.data) 
#' #Executing this command will yield the following data frame:
#' #  Ben   Gerry n
#' # <chr> <chr> <chr>
#' #   a     b     1    
#' #   a     d     1
#' #   b     b     2    
#' #   c     c     3    
#' #   d     b     1    
#' #   d     d     1    
#' #   e     e     1    
#' #   a     a     0
#' @export
freq.supp.fn <- function(freq.data,categories.vec){
  cnames <- colnames(freq.data)
  v1.supp <- NULL
  v2.supp <- NULL
  freq.data.all <- freq.data
  r1categ <- sort(as.character(unique(na.omit(freq.data[[1]]))))
  r2categ <- sort(as.character(unique(na.omit(freq.data[[2]]))))
  if (missing(categories.vec)) categories.vec<-""
  else categories.vec <- sort(as.character(categories.vec))
  if (!identical(categories.vec,r1categ)){
    v1.supp <- setdiff(categories.vec,r1categ)
    n.v2supp<-length(v1.supp)
    freq.v1 <- as.data.frame(cbind(v1.supp,r2categ[1:n.v2supp],rep(0,n.v2supp)))
    freq.v1 <- setNames(as_tibble(freq.v1),cnames)
    freq.v1[[3]] <- as.integer(as.character(freq.v1[[3]]))
  }
  if (!identical(categories.vec,r2categ)){
    v2.supp <- setdiff(categories.vec,r2categ)
    n.v3supp <- length(v2.supp)
    freq.v2 <- as.data.frame(cbind(r1categ[1:n.v3supp],v2.supp,rep(0,n.v3supp)))
    freq.v2 <- setNames(as_tibble(freq.v2),cnames)
    freq.v2[[3]] <- as.integer(as.character(freq.v2[[3]]))
  }
  if (length(v1.supp)>0) freq.data.all <- rbind(freq.data.all,freq.v1)
  if (length(v2.supp)>0) freq.data.all <- rbind(freq.data.all,freq.v2)
  return(freq.data.all)
}

#' Dataset representing a 10x10 contingency table
#' 
#' This dataset contains a 10x10 contingency table.
#' 
#' @format The first column is alphabetic whereas the remaining columns contains
#' integers.
#'  \describe{
#'     \item{Categories}{Contains category's name C1,C2, ..., C10)}
#'     \item{C1}{Colum C1}
#'     \item{C2}{Colum C2}
#'     \item{C3}{Colum C3}
#'     \item{C4}{Colum C4}
#'     \item{C5}{Colum C5}
#'     \item{C6}{Colum C6}
#'     \item{C7}{Colum C7}
#'     \item{C8}{Colum C8}
#'     \item{C9}{Colum C9}
#'     \item{C10}{Colum C10}
#'  }   
#'  
#' @source K. Gwet, PhD.
#' 
"agree.contingency"

#' Ratings of 12 units from 2 raters named Ben and Gerry
#' 
#' This dataset contains ratings that 2 raters named Ben and Gerry assigned to 12 units distributed in 2 groups 
#' "G1" and "G2".
#' @format Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
#'  \describe{
#'     \item{Group}{Group Name}
#'     \item{Units}{Unit number}
#'     \item{Ben}{Ben's Ratings}
#'     \item{Gerry}{Gerry's Ratings}
#'  }   
#' The first 2 columns "Group" and "Units" play a descriptive role here and are not used by any fucntion included in this
#' package. One will typically use cac.ben.gerry[,c(3,4)] or cac.ben.gerry[,c("Ben","Gerry")] as input 
#' dataset.
"cac.ben.gerry"

---
title: "irrCAC-benchmarking"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{irrCAC-benchmarking}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(irrCAC)
```

# Abstract

The **irrCAC** is an R package that provides several functions for calculating various chance-corrected agreement coefficients (see [overview.html](overview.html) for ageneral overview), their weighted versions using various weights (see [weighting.html](weighting.html) for a more detailed discusssion on the weighting of agreement coefficients). In this document, I like to show you how you would implement the benchmarking approach discussed in chapter 6 of Gwet (2014). This package closely follows the general framework of inter-rater reliability assessment presented by Gwet (2014).

In a nutshell, the problem consists of qualifying the magnitude of a given agreement coeffficient as *poor*, *good*, *very good*, or something else.  There are essentially 3 bechmarking scales that have been mentioned in the inter-rater reliability, and which are covered in this package. These are the **Landis-Koch**, **Altman**, and **Fleiss** bechmarking scales defined as follows:
```{r}
landis.koch
altman
fleiss
```
These are data frames that become available to you as soon as you you install the irrCAC package.  

#
# Interpreting the magnitude of agreement coeeficients

Suppose that you computed Gwet's $\mbox{AC}_1$ coefficient using raw ratings from the dataset **cac.raw4raters**. You now want to qualify the magnitude of this coefficient using one of the benchmarking scales. Although you would normally choose one of the 3 benchmarking scales, I will use all 3 for illustration purposes.  You would proceed as follows:
```{r}
  ac1 <- gwet.ac1.raw(cac.raw4raters)$est
  data.frame(ac1$coeff.val, ac1$coeff.se)
  landis.koch.bf(ac1$coeff.val, ac1$coeff.se)
  altman.bf(ac1$coeff.val, ac1$coeff.se)
  fleiss.bf(ac1$coeff.val, ac1$coeff.se)
```
Each of the functions *landis.koch.bf(ac1\$coeff.val, ac1\$coeff.se)*, *altman.bf(ac1\$coeff.val, ac1\$coeff.se)*, and *fleiss.bf(ac1\$coeff.val, ac1\$coeff.se)* produces 2 columns: the agreement strength level, and the associated cumulative membership probability (*CumProb*). *CumProb* represents the probability that the true agreement strength level is the one associated with the probability or one that is better. I recommended in Gwet (2014) to retain an agreement strength level that is associated with *ComProb* that exceeds 0.95. 
* Landis-Koch Scale
  Since $\mbox{AC}_1=0.775$ then according to the Landoch-Koch benchmarking scale, this 
  agreement coefficient is deemed *Moderate* since the associated *CumProb* exceeds the 
  thereshold of 0.95. It cannot be qualified as *substantial* due to its standard error 
  of 0.14295 being high. See chapter 4 of Gwet (2014) for a more detailed discussion on this topic. 
  
* Altman Scale
  According to Altman's benchmarking scale, the $\mbox{AC}_1$ agreement coefficient of 0.775 is
  *Moderate* as well. 
  
* Fleiss Scale
  According to Fleiss' benchmarking scale, the $\mbox{AC}_1$ agreement coefficient of 0.775 is
  *Intermediate to Good*. 

#
# References:
1. Gwet, K.L. (2014). "*Handbook of Inter-Rater Reliability*," 4th Edition. Advanced Analytics, LLC

---
title: "Weighted Chance-corrected Agreement Coefficients"
author: "Kilem L. Gwet, Ph.D."
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{irrCAC: Computing weighted agreement coefficients}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(irrCAC)
```

# Abstract

The **irrCAC** is an R package that provides several functions for calculating various chance-corrected agreement coefficients. This package closely follows the general framework of inter-rater reliability assessment presented by Gwet (2014). The document [overview.html](overview.html) provides a broad overview of the different ways you may compute various agreement coefficients using the irrCAC package.  However, it is common for researchers to want to compute the weighted versions of agreement coefficients in order to account for the ordinal nature of some ratings.

This document shows you how to specify the weights to comnpute the weighted kappa coefficient, the weighted $\mbox{AC}_2$ coefficient and many others. 

#
# The different weights 

A set of weights to be applied to an agreement is generally defined in the form a matrix (i.e. a square table) whose dimension
is defined par the number of categories into which the rater can classify each subject.  Although the user can define her own custom weights, the inter-rater reliability literature offers predefined weights, which can be generated using functions in this package as follows:
```{r}
  identity.weights(1:3)
  radical.weights(1:3)
  linear.weights(1:3)
  ordinal.weights(1:3)
  quadratic.weights(1:3)
  circular.weights(1:3)
  bipolar.weights(1:3)
```
You may need to read chapter 3 of Gwet (2014) for a more detailed discussion of weighted agreement coefficients. Identifty weights yield unweighted coefficients and the most commonly-used weights are the quadratic and linear weights.

#
# Weighted Agreement Coefficients

I will show here how to compute weighted agreement coefficients separately for each of the 3 ways of organizing your input ratings that are covered in this package.  

1. *The contingency table*: this dataset describes ratings from 2 raters only in such a way that each row represents a rating used by rater 1 and each column a rating used by rater 2 (see **cont3x3abstractors**) for an example of such a dataset).   

1. *The dataset of raw ratings*: this dataset is a listing of all ratings assigned each individual subject by each rater.  The row represents one subject, the column represents one rater and each value is the rating that the rater assigned to the subject (you may view the package dataset **cac.raw4raters** as an example).

1. *The distribution of raters*: each row of this dataset represents a subject, each column a rating used by any of the raters and each dataset value is the the number of raters who assigned the specified rating to the subject (you may see **distrib.6raters** as an example)  

## Weighting ratings from a contingency table

Suppose you want to use quadratic weights to compute various weighted coefficients using the dataset *cont3x3abstractors*. Using the *quadratic.weights* function presented above, you would proceed as follows:
```{r}
  cont3x3abstractors
  q <- nrow(cont3x3abstractors)
  kappa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  scott2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  gwet.ac1.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  bp2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  krippen2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
  pa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))
```

* You may use any of the weight generating functions previously discusssed to change your weights.

* You may also supply your own custom set of weights to these functions.  No need to stick to the weights recommended in the inter-rater reliability literature. 

## Weighting for a dataset of raw ratings

Suppose you want to use quadratic weights to compute various weighted coefficients using the dataset cac.raw4raters. You would proceed as follows:
```{r}
  pa.coeff.raw(cac.raw4raters,weights = "quadratic")$est
  gwet.ac1.raw(cac.raw4raters,weights = "quadratic")$est
  fleiss.kappa.raw(cac.raw4raters,weights = "quadratic")$est
  krippen.alpha.raw(cac.raw4raters,weights = "quadratic")$est
  conger.kappa.raw(cac.raw4raters,weights = "quadratic")$est
  bp.coeff.raw(cac.raw4raters,weights = "quadratic")$est
```

* If you want to use a set of weights other than quadratic weights you may replace "quadratic" with one of the following strings of characters: "ordinal", "linear", "radical", "ratio", "circular", or "bipolar".

* You may also supply your own custom set of weights to these functions.  In this case, the value of  the **weight** parameter will not be a string of characters.  Instead, it will be an actual matrix containing the weight values you have decided to use. 


## Weighting when input data is the distribution of raters by subject and rating

The calculation of weighted agreement coefficients for this type of datasets is very similar to the calculation of agreement coefficients with contingency tables. You need to use the weight-generating functions such as the *quadratic.weights()* fucntion to which the vector of ratings is supplied as parameter.  As an example, suppose you want to compute various weighted agreement coefficients using quadratic weights and the *distrib.6raters* dataset.  This would be accomplished as follows:  
```{r}
  q <- ncol(distrib.6raters)
  gwet.ac1.dist(distrib.6raters,weights = quadratic.weights(1:q))
  fleiss.kappa.dist(distrib.6raters,weights = quadratic.weights(1:q))
  krippen.alpha.dist(distrib.6raters,weights = quadratic.weights(1:q))
  bp.coeff.dist(distrib.6raters,weights = quadratic.weights(1:q))
```

# References:
1. Gwet, K.L. (2014). "*Handbook of Inter-Rater Reliability*," 4th Edition. Advanced Analytics, LLC

---
title: "Calculating Chance-corrected Agreement Coefficients (CAC)"
author: "Kilem L. Gwet, Ph.D."
date: "`r Sys.Date()`"
output: 
  html_document: default
  knitr:::html_vignette:
  pdf_document: default
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{irrCAC: Calculating Chance-corrected Agreement Coefficients (CAC)}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(irrCAC)
```

# Abstract

The **irrCAC** is an R package that provides several functions for calculating various chance-corrected agreement coefficients. This package closely follows the general framework of inter-rater reliability assessment presented by Gwet (2014). A similar package was developed for STATA users by Klein (2018).


The functions included in this package can handle 3 types of input data: (1) the contingency table, (2) the distribution
of raters by subject and by category, (3) the raw data, which is essentially a plain dataset where each row represents 
a subject and each column, the ratings associated with one rater. The list of all datasets containined in this package can
be listed as follows:

```{r}
  data(package="irrCAC")
```

# Computing Agreement Coefficients

##  Computing agreement Coefficients from Contingency tables

**cont3x3abstractors** is one of 2 datasets included in this package and that contain rating data from 2 raters organized in the form of a contingency table. The following r script shows how to compute Cohen's kappa, Scott's Pi, Gwet's $\mbox{AC}_1$, Brennan-Prediger, Krippendorff's alpha, Bangdiwala's B, and the percent agreement coefficients from this dataset.
```{r}
  cont3x3abstractors
  kappa2.table(cont3x3abstractors)
  scott2.table(cont3x3abstractors)
  gwet.ac1.table(cont3x3abstractors)
  bp2.table(cont3x3abstractors)
  krippen2.table(cont3x3abstractors)
  bangdiwala.table(cont3x3abstractors)
  pa2.table(cont3x3abstractors)
```
Suppose that you only want to obtain Gwet's $\mbox{AC}_1$ coefficient, but don't care about the associated precision measures such as the standard error, confidence intervals or p-values. You can accomplish this as follows:  
```{r} 
  ac1 <- gwet.ac1.table(cont3x3abstractors)$coeff.val
```
Then use the variable ac1 to obtain $\mbox{AC}_1=`r round(ac1,3)`$.


Another contingency table included in this package is named **cont4x4diagnosis**.  You may use it to experiment with the r functions listed above. If you are interested in the weighted agreement coefficients, you can find more information in [weighting.html](weighting.html). Note that Bangdiwala's weighted coefficient is not implemented in this package.


## Computing agreement coefficients from the distribution of raters by subject & category

Included in this package is a small dataset named **distrib.6raters**, which contains the distribution of 6 raters by subject and category. Each row represents a subject (i.e.  a psychiatric patient) and the number of raters (i.e. psychiatrists) who classified it into each category used in the inter-rater reliability study. Here is the dataset and how it can be used to compute the various agreement coefficients:
```{r}
distrib.6raters
gwet.ac1.dist(distrib.6raters)
fleiss.kappa.dist(distrib.6raters)
krippen.alpha.dist(distrib.6raters)
bp.coeff.dist(distrib.6raters)
```
Once again, you can request a single value from these functions. To get only Krippendorff's alpha coefficient without it's precission measures, you may proceed as follows:
```{r} 
  alpha <- krippen.alpha.dist(distrib.6raters)$coeff
```
The newly-created alpha variable gives the coefficient $\alpha = `r alpha`$. 

Two additional datasets that represent ratings in the form of a distribution of raters by subject and by category, are included in this package. These datasets are **cac.dist4cat** and **cac.dist4cat**. Note that these 2 datasets contain more columns than needed to run the 4 functions presented in this section. Therefore, the columns associated with the response categories must be extracted from the original datasets before running the functions. For example, computing Gwet's $\mbox{AC}_1$ coefficient using the **cac.dist4cat** dataset should be done as follows:
```{r} 
  ac1 <- gwet.ac1.dist(cac.dist4cat[,2:4])$coeff
``` 

Note that the input dataset supplied to the \emph{gwet.ac1.dist} function is **cac.dist4cat[,2:4]**. That is, only columns 2, 3, and 4 are extracted from the original datset and used as input data. We know from the value of the newly created variable \emph{ac1} that $\mbox{AC}_1=`r ac1`$.

## Computing agreement coefficients from raw ratings

One example dataset of raw ratings included in this package is **cac.raw4raters** and looks like this:
```{r}
  cac.raw4raters
```
As you can see, a dataset of raw ratings is merely a listing of ratings that the raters assigned to the subjects.  Each row is associated with a single subject.Typically, the same subject would be rated by all or some of the raters.  The dataset **cac.raw4raters** contains some missing ratings represented by the symbol NA, suggesting that some raters did not rate all subjects. As a matter of fact, in this particular case, no rater rated all subjects.  

Here is you can compute the various agreement coefficients using the raw ratings:
```{r}
pa.coeff.raw(cac.raw4raters)
gwet.ac1.raw(cac.raw4raters)
fleiss.kappa.raw(cac.raw4raters)
krippen.alpha.raw(cac.raw4raters)
conger.kappa.raw(cac.raw4raters)
bp.coeff.raw(cac.raw4raters)
```
Most users of this package will only be interessted in the agreement coefficients and possibly in the related statistics such as the standard error and p-values.  In this case, you should run these functions as follows ($\mbox{AC}_1$ is used here as an example. Feel free to experiment with the other coefficients):

```{r}
ac1 <- gwet.ac1.raw(cac.raw4raters)$est
ac1
```
You can even request only the $\mbox{AC}_1$ coefficient estimate `r ac1$coeff.val`.  You will then proceed as follows:
```{r}
ac1 <- gwet.ac1.raw(cac.raw4raters)$est
ac1$coeff.val
```




# References:
1. Gwet, K.L. (2014). "*Handbook of Inter-Rater Reliability*," 4th Edition. Advanced Analytics, LLC
1. Klein, D. (2018) doi:<https://doi.org/10.1177/1536867X1801800408>. "Implementing a general framework for assessing interrater agreement in Stata," *The Stata Journal* Volume 18, Number 4, pp. 871-901.
1. Shankar, V. and Bangdiwala, S.I. (2014). Observer agreement paradoxes in $2\times2$ tables: Comparison of agreement measures. *BMC Medical Research Methodology*, 14(1). <https://doi.org/10.1186/1471-2288-14-100>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fleiss_doc.R
\docType{data}
\name{fleiss}
\alias{fleiss}
\title{Dataset describing Fleiss' Benchmarking Scale}
\format{
Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
 \describe{
    \item{lb.FL}{The interval lower bound}
    \item{ub.FL}{The interval upper bound}
    \item{interp.FL}{The interpretation}
 }
}
\source{
Fleiss, J. L. (1981). Statistical Methods for Rates and Proportions. John Wiley & Sons.
}
\usage{
fleiss
}
\description{
This dataset contains information describing Fleiss' scale for benchmarking chance-corrected agreement 
coefficients such as Gwet AC1/AC2, Kappa and many others.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff2.r
\name{freq.supp.fn}
\alias{freq.supp.fn}
\title{freq.supp.fn: This function reads a 3-variable input data file containing unique pairs
of categories along with their frequency of occurrences, and outputs a similar file
where all possible pairs of categories are represented, some with a frequency of 
occurrence of 0.}
\usage{
freq.supp.fn(freq.data, categories.vec)
}
\arguments{
\item{freq.data}{The input data file containing all unique combinations of reported 
categories.}

\item{categories.vec}{A vector containing the complete set of categories available to 
raters (e.g. "a", "b", "c", "d", "e"). The raters will not necessarily use all of 
these categories.}
}
\value{
This function returns a complete data frame containing all possible combinations of
of categories in the categories.vec vector. Newly-added combinations of categories will
have a frequency occurrence of 0.
}
\description{
freq.supp.fn: This function reads a 3-variable input data file containing unique pairs
of categories along with their frequency of occurrences, and outputs a similar file
where all possible pairs of categories are represented, some with a frequency of 
occurrence of 0.
}
\examples{
#The dataset "freqs.data" comes with this package. Analyze it as follows:
freq.supp.fn(freqs.data) 
#Executing this command will yield the following data frame:
#  Ben   Gerry n
# <chr> <chr> <chr>
#   a     b     1    
#   a     d     1
#   b     b     2    
#   c     c     3    
#   d     b     1    
#   d     d     1    
#   e     e     1    
#   a     a     0
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqs.data_doc.R
\docType{data}
\name{freqs.data}
\alias{freqs.data}
\title{Distribution of 10 subjects by rater (Ben and Gerry) and by category.}
\format{
Each row of this dataset describes a unique pair of categories and the number
of subjects that both Ben and Gerry classified into the 2 categories respectively.
 \describe{
    \item{Ben}{Categories that Ben used to classify the subjects.}
    \item{Gerry}{Categories that Gerry used to classify the subjects}
    \item{n}{Number of subjects that Ben and Gerry classified into the associated pair of categories.}
 }
}
\source{
N/A.
}
\usage{
freqs.data
}
\description{
This dataset contains rating data collected from 10 human subjects by 2 raters named
Ben and Gerry. While the 5 categories a, b, c, d, and e are available for rating, Gerry
never used category a.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/landis.koch_doc.R
\docType{data}
\name{landis.koch}
\alias{landis.koch}
\title{Dataset describing the Landis & Koch Benchmarking Scale}
\format{
Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
 \describe{
    \item{lb.LK}{The interval lower bound}
    \item{ub.LK}{The interval upper bound}
    \item{interp.LK}{The interpretation}
 }
}
\source{
Landis, J.R. & Koch G. (1977). The measurement of observer agreement for categorical data, \emph{Biometrics}, 
33, 159-174.
}
\usage{
landis.koch
}
\description{
This dataset contains information describing the Landis & Koch scale for benchmarking chance-corrected agreement 
coefficients such as Gwet AC1/AC2, Kappa and many others.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.raw.g1g2_doc.R
\docType{data}
\name{cac.raw.g1g2}
\alias{cac.raw.g1g2}
\title{Dataset of raw ratings from 4 Raters on 14 Subjects that belong to 2 groups named "G1" and "G2"}
\format{
This dataset contains ratings obtained from an experiment where 4 raters classified 14 subjects into 5 
possible categories labeled as a, b, c, d, and e. None of the 4 raters scored all 14 units. Therefore, 
some missing ratings appear in each of the columns associated with the 4 raters.

Note that only the the 4 last columns are to be used with the functions included in this package.  The first 2
columns only play a descriptive role and are not used in any calculation.
 \describe{
    \item{Group}{This variable repsents the unit number.}
    \item{Units}{This variable repsents the unit number.}
    \item{Rater1}{All ratings from rater 1}
    \item{Rater2}{All ratings from rater 2}
    \item{Rater3}{All ratings from rater 3}
    \item{Rater4}{All ratings from rater 4}
 }
}
\usage{
cac.raw.g1g2
}
\description{
This dataset contains data from a reliability experiment where 4 raters identified as Rater1, Rater2, Rater3 and Rater4
scored 14 units on a 5-point alphabetical scale based on the values a, b, c, d and e. These 14 units are allocated to 
2 groups named G1 and G2.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.dist4cat_doc.R
\docType{data}
\name{cac.dist4cat}
\alias{cac.dist4cat}
\title{Distribution of 4 raters by Category and Subject - Subjects allocated in 2 groups A and B.}
\format{
This dataset has 15 rows (for the 15 subjects) and 4 columns. Only the last 3 columns representing the
categories into which subjects are classified are used in the calculations - unless the sub-group analysis is required.    
 \describe{
    \item{Group}{This variable repsents the subject number.}
    \item{a}{category a}
    \item{b}{Category b}
    \item{c}{Category c}
 }
}
\usage{
cac.dist4cat
}
\description{
This dataset summarizes the ratings assigned by 4 raters who classified 15 subjects into one of 3 categories
named "a", "b", and "c".
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.raw.gender_doc.R
\docType{data}
\name{cac.raw.gender}
\alias{cac.raw.gender}
\title{Rating Data from 4 Raters and 15 human Subjects, 9 of whom are female and 6 males.}
\format{
This dataset contains ratings obtained from an experiment where 4 raters classiffied 15 subjects into 3 
possible categories labeled as a, b, and c.

Note that only the the 4 last columns are to be used with the functions included in this package.  The first 
column only plays a descriptive role and is not to be used in any calculation.
 \describe{
    \item{Group}{This variable repsents the unit number.}
    \item{RaterA}{All ratings from rater 1}
    \item{RaterB}{All ratings from rater 2}
    \item{RaterC}{All ratings from rater 3}
    \item{RaterD}{All ratings from rater 4}
 }
}
\usage{
cac.raw.gender
}
\description{
This dataset contains data from a reliability experiment where 4 raters scored 15 units on a 3-point alphabetic scale 
based on the values a, b, and c.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff2.r
\name{scott2.table}
\alias{scott2.table}
\title{Scott's coefficient for 2 raters}
\usage{
scott2.table(
  ratings,
  weights = identity.weights(1:ncol(ratings)),
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{A square table of ratings (assume no missing ratings).}

\item{weights}{An optional matrix that contains the weights used in the weighted analysis. By default, this parameter contaings the identity weight matrix, which leads to the unweighted analysis.}

\item{conflev}{An optional parameter that specifies the confidence level used for constructing confidence intervals. By default the function assumes the standard value of 95\%.}

\item{N}{An optional parameter representing the finite population size if any. It is used to perform the finite population correction to the standard error. It's default value is infinity.}
}
\value{
A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
}
\description{
Scott's coefficient for 2 raters
}
\examples{
#The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
scott2.table(cont3x3abstractors) #Yields Scott's Pi coefficient along with precision measures
scott <- scott2.table(cont3x3abstractors)$coeff.val #Yields Scott's coefficient alone.
scott
q <- nrow(cont3x3abstractors) #Number of categories
scott2.table(cont3x3abstractors,weights = quadratic.weights(1:q)) #weighted Scott's coefficient
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff2.r
\name{long2wide.fn}
\alias{long2wide.fn}
\title{long2wide.fn: This function transforms a 3-column dataset of frequencies to a square
matrix or a contingency table. This function uses the freq.supp.fn() function.}
\usage{
long2wide.fn(freqs.long)
}
\arguments{
\item{freqs.long}{A 3-column data frame, where the first 2 variables represent the categories
that both raters have actually used when classifying the subjects. The third and last 
variable is generally named "n" and represents the count of subjects that classified into
the 2 associated categories by both raters.}
}
\value{
A matrix that represents a contingency showing the distribution of subjects by
rater and category.
}
\description{
long2wide.fn: This function transforms a 3-column dataset of frequencies to a square
matrix or a contingency table. This function uses the freq.supp.fn() function.
}
\examples{
#The dataset "freqs.data" comes with this package. Analyze it as follows:
long2wide.fn(freqs.data) #Yields a 5x5 matrix
#This will produce the following 5x5 matrix:
#> long2wide.fn(freqs.data)
#a b c d e
#a 0 1 0 1 0
#b 0 2 0 0 0
#c 0 0 3 0 0
#d 0 1 0 1 0
#e 0 0 0 0 1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.raw2raters_doc.R
\docType{data}
\name{cac.raw2raters}
\alias{cac.raw2raters}
\title{Dataset of raw ratings by 2 raters and 12 subjects.}
\format{
This dataset contains ratings obtained from an experiment where 2 
raters classified 12 subjects into 3 categories A, B, and C. Two of the 12
subjects were rated by a single rater. Consequently, this dataset contains
2 missing ratings that are identified with the symbol <NA>.

 \describe{
    \item{rater1}{All ratings from rater 1}
    \item{rater2}{All ratings from rater 2}
 }
}
\source{
Gwet, K.L. (2021) \emph{Handbook of Inter-Rater Reliability: 
Volume 1}, 5th Edition. AgreeStat Analytics.
}
\usage{
cac.raw2raters
}
\description{
This dataset contains 12 rows and 2 columns. Each row represents a subject
and the 2 columns labeled as "rater1" and "rater2" contain the ratings 
produced by the 2 raters.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distrib.6raters_doc.R
\docType{data}
\name{distrib.6raters}
\alias{distrib.6raters}
\title{Distribution of 6 psychiatrists by Subject/patient and diagnosis Category.}
\format{
This dataset has 15 rows (for the 15 subjects) and 7 columns. Only the last 6 columns representing the
categories into which subjects are classified are used in the calculations.    
 \describe{
    \item{Subject}{This variable repsents the subject number.}
    \item{Personality.Disorder}{Personality disorder category}
    \item{Schizophrenia}{Schizophrenia Category}
    \item{Neurosis}{Neurosis category}
    \item{Other}{"Other" category}
 }
}
\source{
Fleiss, J. L. (1971). Measuring nominal scale agreement among many raters, \emph{Psychological Bulletin}, 76, 
378-382.
}
\usage{
distrib.6raters
}
\description{
This dataset summarizes the ratings assigned by 6 psychiatrists classifying 15 patients into one of five categories
named "Depression", "Personal Disorder", "Schizophrenia", "Neurosis" and "Other".
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.dist.g1g2_doc.R
\docType{data}
\name{cac.dist.g1g2}
\alias{cac.dist.g1g2}
\title{Distribution of 4 raters by subject and by category, for 14 Subjects that belong to 2 groups "G1" and "G2"}
\format{
This dataset contains ratings obtained from an experiment where 4 raters classified 14 subjects into 5 
possible categories labeled as a, b, c, d, and e. None of the 4 raters scored all 14 units. Therefore, 
some missing ratings appear in each of the columns associated with the 4 raters.

Note that only the the 4 last columns are to be used with the functions included in this package.  The first 2
columns only play a descriptive role and are not used in any calculation.
 \describe{
    \item{Group}{This variable represents the group name.}
    \item{Units}{This variable represents the unit number.}
    \item{a}{Number of raters who classified the subject represented by the row into category "a"}
    \item{b}{Number of raters who classified the subject represented by the row into category "b"}
    \item{c}{Number of raters who classified the subject represented by the row into category "c"}
    \item{d}{Number of raters who classified the subject represented by the row into category "d"}
    \item{e}{Number of raters who classified the subject represented by the row into category "e"}
 }
}
\usage{
cac.dist.g1g2
}
\description{
This dataset contains rating data in the form of a subject-level distribution of 4 raters by category the subject
was classified into. A total of 4 raters had to classify 14 subjects into one of 5 categories "a", "b", "c", "d", and "e". 
This dataset is different version of the more detailed "cac.raw.g1g2" dataset. While "cac.raw.g1g2" tells you
about the exact category into which each rater classified all subjects, "cac.dist.g1g2" on the other hand, can only tell 
you how many raters classified a given subject into a particular category.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff3.raw.r
\name{krippen.alpha.raw}
\alias{krippen.alpha.raw}
\title{Krippendorff's alpha coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.}
\usage{
krippen.alpha.raw(
  ratings,
  weights = "unweighted",
  categ.labels = NULL,
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.}

\item{weights}{is a mandatory parameter that is either a string variable or a matrix. 
The string describes one of the predefined weights and must take one of the values 
("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
If this parameter is a matrix then it must be a square matri qxq where q is the number 
of posssible categories where a subject can be classified. If some of the q possible 
categories are not used, then it is strobgly advised to specify the complete list of 
possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.}

\item{categ.labels}{An optional vector parameter containing the list of all possible ratings. It may be useful in 
case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
calculations.}

\item{conflev}{An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.}

\item{N}{An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity.}
}
\value{
A data list containing 3 objects: (1) a one-row data frame containing various statistics including the 
requested agreement coefficient-in this case, Krippendorff's alpha, (2) the weight matrix used in the calculations if any, and 
(3) the vector of categories used in the analysis. These could be categories reported by the raters, or those that were available 
to the raters whether they used them or not.  The output data frame contains the following variables: "coeff.name" (coefficient 
name), "pa" (the percent agreement), "pe" (the percent chance agreement), coeff.val (Krippendorff's alpha estimate), "coeff.se 
(standard error), conf.int" (Krippendorff alpha's confidence interval),"p.value" (Krippendorff alpha's p-value), "w.name" 
(the weights' identification).
}
\description{
Krippendorff's alpha coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
}
\examples{
#The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
cac.raw4raters
krippen.alpha.raw(cac.raw4raters) #Alpha coeff. , precision measures, weights & categories
krippen.alpha.raw(cac.raw4raters)$est #Krippendorff's alpha with precision measures
alpha <- krippen.alpha.raw(cac.raw4raters)$est$coeff.val #Krippendorff's alpha alone.
alpha
krippen.alpha.raw(cac.raw4raters, weights = "quadratic") #weighted alpha/ quadratic wts
}
\references{
Gwet, K. (2014). \emph{Handbook of Inter-Rater Reliability: The Definitive Guide to Measuring the Extent of Agreement Among Multiple Raters}, 4th Edition.  Advanced Analytics, LLC.\cr\cr
Krippendorff (1970). ``Bivariate agreement coefficients for reliability of data." \emph{Sociological Methodology},2,139-150.\cr\cr
Krippendorff (1980). \emph{Content analysis: An introduction to its methodology} (2nd ed.), New-bury Park, CA: Sage.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff2.r
\name{bangdiwala2RR.fn}
\alias{bangdiwala2RR.fn}
\title{Bangdiwala B coefficient for 2 raters when input dataset is made up of 2 columns of raw data.}
\usage{
bangdiwala2RR.fn(fra.ratings.raw, conflev = 0.95, N = Inf)
}
\arguments{
\item{fra.ratings.raw}{A dataframe with 2 columns of raw ratings.}

\item{conflev}{An optional parameter that specifies the confidence level used for constructing confidence intervals. By default the function assumes the standard value of 95\%.}

\item{N}{An optional parameter representing the finite population size if any. It is used to perform the finite population correction to the standard error. It's default value is infinity.}
}
\value{
A data frame containing the following 9 variables: coeff.name, b1, b2, 
coeff.val, coeff.se, conf.int, p.value, n and name of the weight used.
}
\description{
Bangdiwala B coefficient for 2 raters when input dataset is made up of 2 columns of raw data.
}
\examples{
#The dataset cac.ben.gerry comes with this package. Analyze it as follows:
bangdi <- bangdiwala2RR.fn(cac.ben.gerry[,c(3,4)]) #using only the last 2 columns.
#The result will be following:
c(bangdi$coeff.name,bangdi$pa,bangdi$pe,bangdi$coeff.val,bangdi$coeff.se,
  bangdi$conf.int,bangdi$p.val,bangdi$tot.obs)
#1 Bangdiwala''s B 0.1322314 0.2066116      0.64 0.2158518 (0.159,1) 7.083e-03      11
bangdi$w.name
#1 Identity
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff2.r
\name{kappa2.table}
\alias{kappa2.table}
\title{Kappa coefficient for 2 raters}
\usage{
kappa2.table(
  ratings,
  weights = identity.weights(1:ncol(ratings)),
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{A square or contingency table of ratings (assume no missing ratings). See the 2 datasets 
"cont3x3abstractors" and "cont4x4diagnosis" that come with this package as examples.}

\item{weights}{An optional matrix that contains the weights used in the weighted analysis.}

\item{conflev}{An optional confidence level for confidence intervals. The default value is the traditional 0.95.}

\item{N}{An optional population size.  The default value is infinity.}
}
\value{
A data frame containing the following 5 variables: coeff.name coeff.val coeff.se coeff.ci coeff.pval.
}
\description{
Kappa coefficient for 2 raters
}
\examples{
#The dataset "cont3x3abstractors" comes with this package. Analyze it as follows:
kappa2.table(cont3x3abstractors) #Yields Cohen's kappa along with precision measures
kappa <- kappa2.table(cont3x3abstractors)$coeff.val #Yields Cohen's kappa alone.
kappa
q <- nrow(cont3x3abstractors) #Number of categories
kappa2.table(cont3x3abstractors,weights = quadratic.weights(1:q))#weighted kappa/quadratic wts
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/x.dist10x5_doc.r
\docType{data}
\name{x.dist10x5}
\alias{x.dist10x5}
\title{Dataset of categorical ratings assigned to 10 subjects and presented in the
form of a distribution of 4 raters by subject and category}
\format{
A data frame of 10 rows and 6 columns.
integers.
 \describe{
    \item{subject}{Patient's identifier}
    \item{q1}{Number of raters who classified the subject into category q1}
    \item{q2}{Number of raters who classified the subject into category q2}
    \item{q3}{Number of raters who classified the subject into category q3}
    \item{q4}{Number of raters who classified the subject into category q4}
    \item{q5}{Number of raters who classified the subject into category q5}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
x.dist10x5
}
\description{
This dataset shows how 4 raters classified 10 subjects into 5 categories
labeled as q1, q2, q3, q4 and q5. Each record is associated with a subject
and shows how the 4 raters are distributed across the 5 categories they
assigned the subject to.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.cac3rd_doc.r
\docType{data}
\name{agree.cac3rd}
\alias{agree.cac3rd}
\title{Dataset showing the distribution of 6 raters by psychiatric condition}
\format{
The first column is of character type whereas the remaining columns 
contains integers.
 \describe{
    \item{Gender}{Contains subject's gender (M or F)}
    \item{Depression}{The interval upper bound}
    \item{Pers.Disorder}{Contains the number of raters who labeled the subject as having personal disorder}
    \item{Schizophrenia}{Contains the number of raters who labeled the subject as having schizophrenia}
    \item{Neurosis}{Contains the number of raters who labeled the subject as having Neurosis}
    \item{Other}{Contains the number of raters who labeled the subject as having Other psychiatric condition}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
agree.cac3rd
}
\description{
This dataset contains rating data about 15 subjects or patients whose 
psychiatric condition was evaluated by 6 raters. Each column represents one 
condition and contains the number of raters who diagnosed the patient with
that condition.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.raw4raters_doc.R
\docType{data}
\name{cac.raw4raters}
\alias{cac.raw4raters}
\title{Rating Data from 4 Raters and 12 Subjects.}
\format{
This dataset contains ratings obtained from an experiment where 4 raters classified 12 subjects into 5 
possible categories labeled as 1, 2, 3, 4, and 5. None of the 4 raters scored all 12 units. Therefore, 
some missing ratings in the form of "NA" appear in each of the columns associated with the 4 raters.

Note that only the the 4 last columns are to be used with the functions included in this package.  The first 
column only plays a descriptive role and is not used in any calculation.
 \describe{
    \item{Units}{This variable repsents the unit number.}
    \item{Rater1}{All ratings from rater 1}
    \item{Rater2}{All ratings from rater 2}
    \item{Rater3}{All ratings from rater 3}
    \item{Rater4}{All ratings from rater 4}
 }
}
\source{
Gwet, K.L. (2014) \emph{Handbook of Inter-Rater Reliability}, 4th Edition, page #120. Advanced Analytics, LLC.
}
\usage{
cac.raw4raters
}
\description{
This dataset contains data from a reliability experiment where 5 observers scored 15 units on a 4-point numeric scale 
based on the values 0, 1, 2 and 3.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmarking.r
\name{landis.koch.bf}
\alias{landis.koch.bf}
\title{Computing Landis-Koch Benchmark Scale Membership Probabilities}
\usage{
landis.koch.bf(coeff, se, BenchDF = landis.koch)
}
\arguments{
\item{coeff}{A mandatory parameter representing the estimated value of an agreement coefficient.}

\item{se}{A mandatory parameter representing the agreement coefficient standard error.}

\item{BenchDF}{An optional parameter that is a 3-column data frame containing the Landis & Koch's benchmark scale 
information. The 3 columns are the interval lower bound, upper bound, and their interpretation. The default value 
is a small file contained in the package and named \emph{landis.koch.RData}, which describes the official 
Landis & Koch's scale intervals and their interpretation.}
}
\value{
A one-column matrix containing the membership probabilities (c.f. \url{https://agreestat.com/papers/inter-rater\%20reliability\%20study\%20design1.pdf})
}
\description{
Computing Landis-Koch Benchmark Scale Membership Probabilities
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff3.dist.r
\name{fleiss.kappa.dist}
\alias{fleiss.kappa.dist}
\title{Fleiss' agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.}
\source{
Fleiss, J. L. (1981). \emph{Statistical Methods for Rates and Proportions}. John Wiley & Sons.
}
\usage{
fleiss.kappa.dist(
  ratings,
  weights = "unweighted",
  categ = NULL,
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{An \emph{nxq} matrix / data frame containing the distribution of raters by subject and category. Each cell \emph{(i,k)} contains the number of raters who classsified subject \emph{i} into category \emph{k}.}

\item{weights}{is an optional parameter that is either a string variable or a matrix. The string describes one of the predefined 
weights and must take one of the values ("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
If this parameter is a matrix then it must be a square matri qxq where q is the number of posssible categories where a subject 
can be classified. If some of the q possible categories are not used, then it is strobgly advised to specify the complete list of 
possible categories as a vector in parametr categ. Otherwise, only the categories reported will be used.}

\item{categ}{An optional parameter representing all categories available to raters during the experiment. This parameter may be useful if 
some categories were not used by any rater inspite of being available to the raters.}

\item{conflev}{An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.}

\item{N}{An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity.}
}
\value{
A vector containing the following information: 
pa(the percent agreement),pe(the percent chance agreement),coeff(Fleiss' agreement coefficient),
stderr(the standard error of Fleiss' coefficient),conf.int(the confidence interval of Fleiss Kappa coefficient),
p.value(the p-value of Fleiss' coefficient),coeff.name ("Fleiss").
}
\description{
Fleiss' agreement coefficient among multiple raters (2, 3, +) when the input dataset is the distribution of raters by subject and category.
}
\examples{
#The dataset "distrib.6raters" comes with this package. It represents the distribution of 6 raters 
#by subject and by category. Note that each row of this dataset sums to the number of raters, which
#is 6. You may this dataset as follows:
distrib.6raters
fleiss.kappa.dist(distrib.6raters) #Fleiss' kappa, precision measures, weights & list of categories
fleiss <- fleiss.kappa.dist(distrib.6raters)$coeff #Yields Fleiss' kappa alone.
fleiss
q <- ncol(distrib.6raters) #Number of categories
fleiss.kappa.dist(distrib.6raters,weights = quadratic.weights(1:q)) #Weighted fleiss/quadratic wts
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.gen.r
\name{quadratic.weights}
\alias{quadratic.weights}
\title{Function for computing the Quadratic Weights}
\usage{
quadratic.weights(categ)
}
\arguments{
\item{categ}{A mandatory parameter representing the vector of all possible ratings.}
}
\value{
A square matrix of quadratic weights to be used for calculating the weighted coefficients.
}
\description{
Function for computing the Quadratic Weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff3.raw.r
\name{pa.coeff.raw}
\alias{pa.coeff.raw}
\title{Percent agreement among multiple raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.}
\usage{
pa.coeff.raw(
  ratings,
  weights = "unweighted",
  categ.labels = NULL,
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.}

\item{weights}{is a mandatory parameter that is either a string variable or a matrix. 
The string describes one of the predefined weights and must take one of the values 
("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
If this parameter is a matrix then it must be a square matri qxq where q is the number 
of posssible categories where a subject can be classified. If some of the q possible 
categories are not used, then it is strobgly advised to specify the complete list of 
possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.}

\item{categ.labels}{An optional vector parameter containing the list of all possible ratings. It may be useful 
in case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
calculations.}

\item{conflev}{An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.}

\item{N}{An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity.}
}
\value{
A data list containing 3 objects: (1) a one-row data frame containing the estimates, (2) the weight matrix 
used in the calculations, and (3) the categories used in the analysis.  The data frame of estimates contains the 
following variables "coeff.name" (coefficient name), "pa" (the percent agreement), "pe" (percent chance-agreement-always equals 0),
"coeff.val" (agreement coefficient = pa), coeff.se (the percent agreement standard error), "conf.int" (the percent agreement confidence interval),
"p.value"(the percent agreement p-value), "w.name"(the weights' identification).
}
\description{
Percent agreement among multiple raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
}
\examples{
#The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
cac.raw4raters
pa.coeff.raw(cac.raw4raters) #Percent agreement, precision measures, weights & categories
pa.coeff.raw(cac.raw4raters)$est #Yields percent agreement with precision measures
pa <- pa.coeff.raw(cac.raw4raters)$est$coeff.val #Yields percent agreement alone.
pa
pa.coeff.raw(cac.raw4raters, weights = "quadratic") #weighted percent agreement/quadratic weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/x.raw12x4_doc.r
\docType{data}
\name{x.raw12x4}
\alias{x.raw12x4}
\title{This dataset contains raw categorical ratings that 4 raters assigned to
12 subjects.}
\format{
A data frame of 12 rows and 5 columns containing integer values 
1, 2, 3, 4 and 5. Missing data points are represented with a dot (".").
integers.
 \describe{
    \item{Units}{Patient's identifier}
    \item{Rater1}{Category into which Rater1 classified the unit}
    \item{Rater2}{Category into which Rater2 classified the unit}
    \item{Rater3}{Category into which Rater3 classified the unit}
    \item{Rater4}{Category into which Rater4 classified the unit}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
x.raw12x4
}
\description{
Dataset of raw ratings assigned to 12 units by 4 raters. Each row is 
associated with a unit identifier and each column to a rater. The columns
are named Rater1, Rater2, Rater3 and Rater4 and contain the categories into
which the unit was assigned. Category values are 1, 2, 3, 4, 5 and some 
ratings are missing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/x.dist6x5psy_doc.r
\docType{data}
\name{x.dist6x5psy}
\alias{x.dist6x5psy}
\title{Dataset showing how 6 psychiatrists classified 15 patients by their mental
health condition.}
\format{
A data frame of 15 rows and 6 columns.
integers.
 \describe{
    \item{Subject}{Patient's identifier}
    \item{Depression}{Number of raters to have diagnosed the patient with depression}
    \item{Personality.Disorder}{Number of raters to have diagnosed the patient with personality disorder}
    \item{Schizophrenia}{Number of raters to have diagnosed the patient with schizophrenia}
    \item{Neurosis}{Number of raters to have diagnosed the patient with neurosis}
    \item{Other}{Number of raters to have diagnosed the patient with other condition}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
x.dist6x5psy
}
\description{
Dataset of 15 psychiatric patients where each row identifies a patient and
shows how the 6 psychiatrists distrubte across 5 mental conditions. These
5 conditions are Depression, personality disorder, Schizophrenia, Neurosis,
and Other.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.ben.gerry_doc.R
\docType{data}
\name{cac.ben.gerry}
\alias{cac.ben.gerry}
\title{Ratings of 12 units from 2 raters named Ben and Gerry}
\format{
Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
 \describe{
    \item{Group}{Group Name}
    \item{Units}{Unit number}
    \item{Ben}{Ben's Ratings}
    \item{Gerry}{Gerry's Ratings}
 }   
The first 2 columns "Group" and "Units" play a descriptive role here and are not used by any fucntion included in this
package. One will typically use cac.ben.gerry[,c(3,4)] or cac.ben.gerry[,c("Ben","Gerry")] as input 
dataset.
}
\usage{
cac.ben.gerry
}
\description{
This dataset contains ratings that 2 raters named Ben and Gerry assigned to 12 units distributed in 2 groups 
"G1" and "G2".
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff3.raw.r
\name{trim}
\alias{trim}
\title{An r function for trimming leading and trealing blanks}
\usage{
trim(x)
}
\arguments{
\item{x}{is a string variable.}
}
\value{
A string variable where leading and trealing blanks are trimmed.
}
\description{
An r function for trimming leading and trealing blanks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/altman_doc.r
\docType{data}
\name{altman}
\alias{altman}
\title{Dataset describing the Altman's Benchmarking Scale}
\format{
Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
 \describe{
    \item{lb.AL}{The interval lower bound}
    \item{ub.AL}{The interval upper bound}
    \item{interp.AL}{The interpretation}
 }
}
\source{
Altman, D.G. (1991). \emph{Practical Statistics for Medical Research}. Chapman and Hall.
}
\usage{
altman
}
\description{
This dataset contains information describing the Altman scale for benchmarking chance-corrected agreement 
coefficients such as Gwet AC1/AC2, Kappa and many others.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.coeff3.raw.r
\name{conger.kappa.raw}
\alias{conger.kappa.raw}
\title{Conger's generalized kappa coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.}
\usage{
conger.kappa.raw(
  ratings,
  weights = "unweighted",
  categ.labels = NULL,
  conflev = 0.95,
  N = Inf
)
}
\arguments{
\item{ratings}{An nxr matrix / data frame of ratings where each column represents one rater and each row one subject.}

\item{weights}{is a mandatory parameter that is either a string variable or a matrix. 
The string describes one of the predefined weights and must take one of the values 
("quadratic", "ordinal", "linear", "radical", "ratio", "circular", "bipolar"). 
If this parameter is a matrix then it must be a square matri qxq where q is the number 
of posssible categories where a subject can be classified. If some of the q possible 
categories are not used, then it is strobgly advised to specify the complete list of 
possible categories as a vector in parametr categ.labels. Otherwise, the program may not work.}

\item{categ.labels}{An optional vector parameter containing the list of all possible ratings. It may be useful in 
case some of the possibe ratings are not used by any rater, they will still be used when calculating agreement 
coefficients. The default value is NULL. In this case, only categories reported by the raters are used in the
calculations.}

\item{conflev}{An optional parameter representing the confidence level associated with the confidence interval. Its default value is 0.95.}

\item{N}{An optional parameter representing the population size (if any). It may be use to perform the final population correction to the variance.  Its default value is infinity.}
}
\value{
A data list containing 3 objects: (1) a one-row data frame containing various statistics including the requested agreement
coefficient, (2) the weight matrix used in the calculations if any, and (3) A vector of categories used in the analysis. These 
could be categories reported by the raters, or those available to the raters whether they used them or not.  The output data frame
contains the following variables: "coeff.name" (coefficient name), "pa" (the percent agreement), "pe" (the percent chance 
agreement), coeff.val (Conger's Kappa estimate), "coeff.se" (standard error), "conf.int" (Conger Kappa's confidence 
interval), "p.value"(agreement coefficient's p-value), "w.name"(the weights' identification).
}
\description{
Conger's generalized kappa coefficient for an arbitrary number of raters (2, 3, +) when the input data represent the raw ratings reported for each subject and each rater.
}
\examples{
#The dataset "cac.raw4raters" comes with this package. Analyze it as follows:
cac.raw4raters
conger.kappa.raw(cac.raw4raters) #Conger's kappa, precision stats, weights & categories
conger.kappa.raw(cac.raw4raters)$est #Conger's kappa with precision measures
conger <- conger.kappa.raw(cac.raw4raters)$est$coeff.val #Yields Conger's kappa alone.
conger
conger.kappa.raw(cac.raw4raters, weights = "quadratic") #weighted Conger's kappa
}
\references{
Conger, A. J. (1980), ``Integration and Generalization of Kappas for Multiple Raters," \emph{Psychological Bulletin}, 88, 322-328.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cont3x3abstractors_doc.R
\docType{data}
\name{cont3x3abstractors}
\alias{cont3x3abstractors}
\title{Distribution of 100 pregnant women by pregnancy type and by abstractor.}
\format{
Each row of this dataset describes an interval and the interpretation of the magnitude it represents.
 \describe{
    \item{Type}{Pregnancy Type.  This variable is shown here for information only and is never used by any function 
                in the irrCAC package.}
    \item{Ectopic}{Ectopic Pregnancy}
    \item{AIU}{Abnormal Intrauterine Pregnancy}
    \item{NIU}{Normal Intrauterine Pregnancy}
 }
}
\source{
Gwet, K.L. (2014). \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC.
}
\usage{
cont3x3abstractors
}
\description{
This dataset contains pregnancy type data collected from 100 women who entered an Emergency Room with a positive 
pregnancy test and a second condition, which is either abdominal pain or vaginal bleeding. After reviewing their 
medical records, 2 reviewers (also referred to as abstractors) classified them into one of the following three pregnancy
categories: Ectopic Pregnancy (Ectopic), Abnormal Intrauterine pregnancy (AIU) and Normal Intrauterine Pregnancy 
(NIU).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/x.raw10x4_doc.r
\docType{data}
\name{x.raw10x4}
\alias{x.raw10x4}
\title{Raw categorical ratings assigned to 10 subjects by 4 raters}
\format{
A data frame of 10 rows and 5 columns.
integers.
 \describe{
    \item{subject}{Patient's identifier}
    \item{rater1}{Category into which rater1 classified the subject}
    \item{rater2}{Category into which rater2 classified the subject}
    \item{rater3}{Category into which rater3 classified the subject}
    \item{rater4}{Category into which rater4 classified the subject}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
x.raw10x4
}
\description{
Dataset of 10 subjects and the categorical ratings assigned to them by 4 
raters. The 5 categories available to raters are labeled as 1, 2, 3, 4, 5.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agree.contingency_doc.r
\docType{data}
\name{agree.contingency}
\alias{agree.contingency}
\title{Dataset representing a 10x10 contingency table}
\format{
The first column is alphabetic whereas the remaining columns contains
integers.
 \describe{
    \item{Categories}{Contains category's name C1,C2, ..., C10)}
    \item{C1}{Colum C1}
    \item{C2}{Colum C2}
    \item{C3}{Colum C3}
    \item{C4}{Colum C4}
    \item{C5}{Colum C5}
    \item{C6}{Colum C6}
    \item{C7}{Colum C7}
    \item{C8}{Colum C8}
    \item{C9}{Colum C9}
    \item{C10}{Colum C10}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
agree.contingency
}
\description{
This dataset contains a 10x10 contingency table.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cont4x4diagnosis_doc.R
\docType{data}
\name{cont4x4diagnosis}
\alias{cont4x4diagnosis}
\title{Distribution of 223 Psychiatric Patients by Type of of Psychiatric Disorder and Diagnosis Method.}
\format{
This dataset contains a 4x4 squared table. The first column is never used in the calculations and only 
contains row names. Only the last 4 columns are used for computing agreement coefficients. 
 \describe{
    \item{Diagnosis}{Pregnancy Type. This variable is shown here for information only and is never used by any function 
                in the irrCAC package.}
    \item{Schizophrenia}{Ectopic Pregnancy}
    \item{Bipolar.Disorder}{Abnormal Intrauterine Pregnancy}
    \item{Depression}{Normal Intrauterine Pregnancy}
    \item{Other}{Normal Intrauterine Pregnancy}
 }
}
\source{
Gwet, K.L. (2014). \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC.
}
\usage{
cont4x4diagnosis
}
\description{
This dataset shows the distribution of 223 psychiatric patients by diagnosis category and by the method used to 
obtain the diagnosis. The first method named ``Clinical Diagnosis" (also known as ``Facility Diagnosis") is used 
in a service facility (e.g. public hospital, or a community unit) and does not rely on a rigorous application of 
research criteria. The second method known as ``Research Diagnosis" is based on a strict application of research 
criteria. Column 1 contains the diagnosis categories into which patients are classified with Method 1.  The first
row on the other hand, shows categories into which patients are classified with Method 2.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.gen.r
\name{identity.weights}
\alias{identity.weights}
\title{Function for computing the Identity Weights}
\usage{
identity.weights(categ)
}
\arguments{
\item{categ}{A mandatory parameter representing the vector of all possible ratings.}
}
\value{
A square matrix of identity weights to be used for calculating the unweighted coefficients.
}
\description{
Function for computing the Identity Weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agreeCAC_doc.r
\docType{data}
\name{agreeCAC}
\alias{agreeCAC}
\title{Ratings of 15 subjects from 4 raters}
\format{
All 5 columns are of alphabetic type.
 \describe{
    \item{Gender}{Contains the subject's gender (M or F)}
    \item{RaterA}{Contains one of ratings a,b,c produced by RaterA}
    \item{RaterB}{Contains one of ratings a,b,c produced by RaterB}
    \item{RaterC}{Contains one of ratings a,b,c produced by RaterC}
    \item{RaterD}{Contains one of ratings a,b,c produced by RaterD}
 }
}
\source{
K. Gwet, PhD.
}
\usage{
agreeCAC
}
\description{
This dataset contains raw ratings of 15 subjects produced by 4 raters named
as RaterA, RaterB, RaterC and RaterD.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cac.raw5obser_doc.R
\docType{data}
\name{cac.raw5obser}
\alias{cac.raw5obser}
\title{Scores assigned by 5 observers to 20 experimental units.}
\format{
This dataset has 15 rows (for the 15 subjects) and 6 columns. Only the last 5 columns associated with the 5
observers are used in the calculations. Of the 5 observers, only observer 3 scored all 15 units. Therefore, some missing
ratings in the form of "NA" appear in the columns associated with the remaining 4 observers.
 \describe{
    \item{Unit}{This variable repsents the unit number.}
    \item{Observer1}{All ratings from Observer 1}
    \item{Observer2}{All ratings from Observer 2}
    \item{Observer3}{All ratings from Observer 3}
    \item{Observer4}{All ratings from Observer 4}
    \item{Observer5}{All ratings from Observer 5}
 }
}
\source{
Gwet, K.L. (2014) \emph{Handbook of Inter-Rater Reliability}, 4th Edition. Advanced Analytics, LLC. 
\emph{A larger version of this table can be found on page #125}
}
\usage{
cac.raw5obser
}
\description{
This dataset contains data from a reliability experiment where 5 observers scored 15 units on a 4-point numeric scale 
based on the values 0, 1, 2 and 3.
}
\keyword{datasets}
